import { createRequire } from 'module';
import assert from 'assert';
import { ZMQ_PUB_CHANNELS, ZMQ_SUB_CHANNELS } from '../../lib/system/constants.mjs';
import { randomMac, ZmqObserver, ZmqPublisher } from '../utils/index.mjs';
import { Server, createPosition, initRepoData, clearRepoData, findNodeByMac, findTrackableByUid } from './helpers.mjs';
const require = createRequire(import.meta.url);
const DbSeeds = require('../seeds/data.json');

describe('Handle ~pos message and re translate to $pos', function () {
    // ~pos and $pos descriptions: https://redpointpositioning.atlassian.net/wiki/spaces/REDPOINT/pages/1314031083/Platform+Message+Processor#Handling-%E2%80%9Cpos%E2%80%9D-messages
    const zmqObserver = new ZmqObserver();
    const zmqPublisher = new ZmqPublisher();
    const tag = {};
    const trackable = {};
    const sublocation = {};
    const association = {};
    const position = {};

    before(async function () {
        await Server().init();
        await initRepoData();
        zmqObserver.init();
        await zmqPublisher.init(); // init with some delay to init zmq connection
        Object.assign(association, DbSeeds.associations[0]);
        Object.assign(sublocation, DbSeeds.sublocations[0]);
        Object.assign(tag, await findNodeByMac(DbSeeds.associations[0].mac));
        Object.assign(trackable, await findTrackableByUid(DbSeeds.associations[0].trackable_uid));
    });
    after(async function () {
        await clearRepoData();
        zmqObserver.destroy();
        zmqPublisher.destroy();
    });

    it('should extend position from the JSON/BIN converter with trackable/sublocation details', function (done) {
        Object.assign(position, createPosition(association.mac, sublocation.id));

        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg, channel) {
            assert(ZMQ_PUB_CHANNELS.POS === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.POS}`);
            assert(zmqMsg.tk_uid === association.trackable_uid, `Trackable uid is not identified`);
            assert(zmqMsg.ts === position.ts, `Position timestamp not equal to ${position.ts}`);
            assert(zmqMsg.data.x === position.data.x, `Position X not equal to ${position.x}`);
            assert(zmqMsg.data.y === position.data.y, `Position Y not equal to ${position.y}`);
            assert(zmqMsg.data.z === position.data.z, `Position Z not equal to ${position.z}`);
            assert(zmqMsg.data.sl_uid === sublocation.uid, `Sublocation uid is not defined`);
            assert(zmqMsg.data.type === 'UWB', `Position type not equal to UWB`);
            assert(zmqMsg.data.mac === position.mac.toUpperCase(), `MAC address has wrong format`);
            assert(zmqMsg.data.fom === position.data.fom, `"fom" has wrong value`);
            assert(zmqMsg.data.er === null, `"er" has wrong value`);
            assert(zmqMsg.data.slvr_mode === position.data.slvr_mode, `"slvr_mode" has wrong value`);
            assert(zmqMsg.data.inactive === position.data.inactive, `"inactive" has wrong value`);
            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.POS, position);
        });
    });

    it('should see changed trackable position and tag last heard', async function () {
        const updatedTrackable = await findTrackableByUid(trackable.uid);
        const updatedTag = await findNodeByMac(tag.mac);
        assert(!tag.last_heard, 'Initial tag should not have a last_heard');
        assert(!trackable.position, 'Initial trackable should not have a position');
        assert(Number(updatedTrackable.position.x) === position.data.x, `Position X not equal to ${position.x}`);
        assert(Number(updatedTrackable.position.y) === position.data.y, `Position Y not equal to ${position.y}`);
        assert(Number(updatedTrackable.position.z) === position.data.z, `Position Z not equal to ${position.z}`);
        assert(updatedTrackable.position.timestamp === position.ts, `Position timestamp not equal to ${position.ts}`);
        assert(updatedTag.last_heard === position.ts, `Tag last heard not updated`);
    });
    // will not send positions if no tk_uid
    xit('should not extend position with trackable details on not associated trackable', function (done) {
        const sublocation = DbSeeds.sublocations[0];
        const position = createPosition(randomMac(), sublocation.id);

        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg, channel) {
            assert(ZMQ_PUB_CHANNELS.POS === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.POS}`);
            assert(zmqMsg.tk_uid === null, `Trackable uid should not be identified`);
            assert(zmqMsg.ts === position.ts, `Position timestamp not equal to ${position.ts}`);
            assert(zmqMsg.data.x === position.data.x, `Position X not equal to ${position.x}`);
            assert(zmqMsg.data.y === position.data.y, `Position Y not equal to ${position.y}`);
            assert(zmqMsg.data.z === position.data.z, `Position Z not equal to ${position.z}`);
            assert(zmqMsg.data.sl_uid === sublocation.uid, `Sublocation uid is not defined`);
            assert(zmqMsg.data.type === 'UWB', `Position type not equal to UWB`);
            assert(zmqMsg.data.mac === position.mac.toUpperCase(), `MAC address has wrong format`);
            assert(zmqMsg.data.fom === position.fom, `"fom" has wrong value`);
            assert(zmqMsg.data.er === null, `"er" has wrong value`);
            assert(zmqMsg.data.slvr_mode === position.slvr_mode, `"slvr_mode" has wrong value`);
            assert(zmqMsg.data.inactive === position.inactive, `"inactive" has wrong value`);
            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.POS, position);
        });
    });

    it('should not extend position with sublocation details on non existing sublocation', function (done) {
        const association = DbSeeds.associations[0];
        const sublocationId = 2222222;
        const position = createPosition(association.mac, sublocationId);

        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg, channel) {
            assert(ZMQ_PUB_CHANNELS.POS === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.POS}`);
            assert(zmqMsg.tk_uid === association.trackable_uid, `Trackable uid is not identified`);
            assert(zmqMsg.ts === position.ts, `Position timestamp not equal to ${position.ts}`);
            assert(zmqMsg.data.x === position.data.x, `Position X not equal to ${position.x}`);
            assert(zmqMsg.data.y === position.data.y, `Position Y not equal to ${position.y}`);
            assert(zmqMsg.data.z === position.data.z, `Position Z not equal to ${position.z}`);
            assert(zmqMsg.data.sl_uid === null, `Sublocation uid should be not defined`);
            assert(zmqMsg.data.type === 'UWB', `Position type not equal to UWB`);
            assert(zmqMsg.data.mac === position.mac.toUpperCase(), `MAC address has wrong format`);
            assert(zmqMsg.data.fom === position.data.fom, `"fom" has wrong value`);
            assert(zmqMsg.data.er === null, `"er" has wrong value`);
            assert(zmqMsg.data.slvr_mode === position.data.slvr_mode, `"slvr_mode" has wrong value`);
            assert(zmqMsg.data.inactive === position.data.inactive, `"inactive" has wrong value`);
            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.POS, position);
        });
    });

    it('should handle er for 0D slvr_mode', function (done) {
        const association = DbSeeds.associations[0];
        const sublocationId = 2222222;
        const position = createPosition(association.mac, sublocationId);
        position.data.fom = 50;
        position.data.slvr_mode = '0D';

        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg, channel) {

            assert(ZMQ_PUB_CHANNELS.POS === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.POS}`);
            assert(zmqMsg.tk_uid === association.trackable_uid, `Trackable uid is not identified`);
            assert(zmqMsg.ts === position.ts, `Position timestamp not equal to ${position.ts}`);
            assert(zmqMsg.data.x === position.data.x, `Position X not equal to ${position.x}`);
            assert(zmqMsg.data.y === position.data.y, `Position Y not equal to ${position.y}`);
            assert(zmqMsg.data.z === position.data.z, `Position Z not equal to ${position.z}`);
            assert(zmqMsg.data.sl_uid === null, `Sublocation uid should be not defined`);
            assert(zmqMsg.data.type === 'UWB', `Position type not equal to UWB`);
            assert(zmqMsg.data.mac === position.mac.toUpperCase(), `MAC address has wrong format`);
            assert(zmqMsg.data.fom === position.data.fom, `"fom" has wrong value`);
            assert(zmqMsg.data.er === position.data.fom * 128, `"er" has wrong value`);
            assert(zmqMsg.data.slvr_mode === position.data.slvr_mode, `"slvr_mode" has wrong value`);
            assert(zmqMsg.data.inactive === position.data.inactive, `"inactive" has wrong value`);
            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.POS, position);
        });
    });

    it('should handle er for 1D slvr_mode', function (done) {
        const association = DbSeeds.associations[0];
        const sublocationId = 2222222;
        const position = createPosition(association.mac, sublocationId);
        position.fom = 50;
        position.slvr_mode = '1D';

        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg, channel) {
            assert(ZMQ_PUB_CHANNELS.POS === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.POS}`);
            assert(zmqMsg.tk_uid === association.trackable_uid, `Trackable uid is not identified`);
            assert(zmqMsg.ts === position.ts, `Position timestamp not equal to ${position.ts}`);
            assert(zmqMsg.data.x === position.data.x, `Position X not equal to ${position.x}`);
            assert(zmqMsg.data.y === position.data.y, `Position Y not equal to ${position.y}`);
            assert(zmqMsg.data.z === position.data.z, `Position Z not equal to ${position.z}`);
            assert(zmqMsg.data.sl_uid === null, `Sublocation uid should be not defined`);
            assert(zmqMsg.data.type === 'UWB', `Position type not equal to UWB`);
            assert(zmqMsg.data.mac === position.mac.toUpperCase(), `MAC address has wrong format`);
            assert(zmqMsg.data.fom === position.data.fom, `"fom" has wrong value`);
            assert(zmqMsg.data.er === null, `"er" has wrong value`);
            assert(zmqMsg.data.slvr_mode === position.data.slvr_mode, `"slvr_mode" has wrong value`);
            assert(zmqMsg.data.inactive === position.data.inactive, `"inactive" has wrong value`);
            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.POS, position);
        });
    });

    it('should extend position from the JSON/BIN converter without PEX data', function (done) {
        Object.assign(position, createPosition(association.mac, sublocation.id));

        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg /*, channel */) {
            assert(!zmqMsg.data.src, `"src" should not be present.`);
            assert(!zmqMsg.data.spd, `"spd" should not be present.`);
            assert(!zmqMsg.data.ornt, `"ornt" should not be present.`);
            assert(!zmqMsg.data.spd_fom, `"spd_fom" should not be present.`);
            assert(!zmqMsg.data.ornt_fom, `"ornt_fom" should not be present.`);
            assert(!zmqMsg.data.metrics, `"metrics" should not be present.`);
            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.POS, position);
        });
    });

    it('should extend position from the JSON/BIN converter with PEX data', function (done) {
        Object.assign(position, createPosition(association.mac, sublocation.id));
        position.data.src = "42",
        position.data.spd = 30000,
        position.data.ornt = -3141,
        position.data.spd_fom = 1,
        position.data.ornt_fom = 255,
        position.data.metrics = "414243"

        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg /*, channel */) {
            assert(zmqMsg.data.src === position.data.src, `"src" has wrong value`);
            assert(zmqMsg.data.spd === position.data.spd, `"spd" has wrong value`);
            assert(zmqMsg.data.ornt === position.data.ornt, `"ornt" has wrong value`);
            assert(zmqMsg.data.spd_fom === position.data.spd_fom, `"spd_fom" has wrong value`);
            assert(zmqMsg.data.ornt_fom === position.data.ornt_fom, `"ornt_fom" has wrong value`);
            assert(zmqMsg.data.metrics === position.data.metrics, `"metrics" has wrong value`);
            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.POS, position);
        });
    });

    it('should extend position from the JSON/BIN converter with PEX data (zero values)', function (done) {
        Object.assign(position, createPosition(association.mac, sublocation.id));
        position.data.spd = 0,
        position.data.ornt = 0,
        position.data.spd_fom = 0,
        position.data.ornt_fom = 0

        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg /*, channel */) {
            assert(zmqMsg.data.spd === position.data.spd, `"spd" has wrong value`);
            assert(zmqMsg.data.ornt === position.data.ornt, `"ornt" has wrong value`);
            assert(zmqMsg.data.spd_fom === position.data.spd_fom, `"spd_fom" has wrong value`);
            assert(zmqMsg.data.ornt_fom === position.data.ornt_fom, `"ornt_fom" has wrong value`);
            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.POS, position);
        });
    });
});
