import { createRepository } from '@rtls-platform/repository';
import { createLogger, LoggerTypes } from '@rtls-platform/logger';
import * as App from './lib/api/index.mjs';
import * as RestApi from './lib/api/rest-api/index.mjs';
import * as ConfigContainer from './lib/config.cjs';
import { createNotificator, NotificatorTypes } from './lib/infrastructure/index.mjs';
import { createDbConnection } from './lib/db/index.mjs';
import UseCaseBase from './lib/usecases/UseCaseBase.mjs';
import ModelBase from './lib/models/ModelBase.mjs';


export async function main() {
    // Init Infrastructure
    const logger = createLogger({
        type: LoggerTypes.Winston,
        context: {
            logLevel: ConfigContainer.config.logLevel,
            serviceName: ConfigContainer.config.serviceName,
        },
    });
    const notificator = createNotificator({
        type: NotificatorTypes.Zmq,
        config: { url: ConfigContainer.config.zmqPushUrl },
    });

    // Init Repository Layer
    //
    const repository = createRepository({
        repositoryInstance: createDbConnection({
            port: ConfigContainer.config.db.port,
            host: ConfigContainer.config.db.host,
        }),
        prefix: ConfigContainer.config.db.prefix,
        nodeIdLimit: ConfigContainer.config.nodeIdLimit,
        networkIdLimit: ConfigContainer.config.networkIdLimit,
        sublocationIdLimit: ConfigContainer.config.sublocationIdLimit,
    });

    // Init Domain Model Layer
    ModelBase.setRepository(repository);

    // Init Service Layer
    UseCaseBase.setNotificator(notificator);

    // Init Controllers Layer (API)
    App.setLogger(logger);
    RestApi.startServer({
        serverPort: ConfigContainer.config.listenPort,
    });

    // Add Global Unhandled Errors Handlers
    async function exit() {
        await RestApi.stopServer();
        logger.info('Exit');

        process.exit(0);
    }

    process.on('SIGTERM', async () => {
        logger.error('SIGTERM signal caught');
        await exit();
    });

    process.on('SIGINT', async () => {
        logger.error('SIGINT signal caught');
        await exit();
    });

    process.on('unhandledRejection', (error) => {
        logger.error('unhandledRejection', error.stack);
    });

    process.on('uncaughtException', (error) => {
        logger.error('uncaughtException', error.stack);
    });
}
