import assert from 'assert';
import { UID } from '../../lib/utils/index.mjs';
import { ERROR_CODE, ZMQ_OBJECTS, ZMQ_ACTIONS, ZMQ_CHANNELS } from '../../lib/system/constants.mjs';
import { isISOTimeFormat, ZmqObserver, put, post, get, del } from '../utils/index.mjs';
import {
    createAnchor,
    createNodeAssociation,
    createNode,
    checkEntities,
    createSublocation,
    createLocation,
    Server,
} from './helpers.mjs';

describe('Anchors Management API', function () {
    const anchor = {};
    const sublocation = {};
    const sublocationTwo = {};
    const location = {};

    before(async function () {
        await Server().init();
        Object.assign(location, await createLocation(`Location ${UID()}`));
        Object.assign(sublocation, await createSublocation(`Sublocation ${UID()}`, location));
        Object.assign(sublocationTwo, await createSublocation(`Sublocation ${UID()}`, location));
    });

    after(async function () {
        await del(`/api/v3/locations/${location.uid}`); // cascade remove of sublocations
    });

    describe('Get Empty Anchors', function () {
        it('should return empty list of anchors', checkEntities('anchors', []));
        it('should return NotFound error on wrong uid', function (done) {
            const invalidUid = UID();
            get(`/api/v3/anchors/${invalidUid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Anchor with uid ${invalidUid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Create Anchors', function () {
        describe('Positive cases', function() {
            it('should create anchor', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: false,
                    position: {
                        x: 1000,
                        y: 2000,
                        z: 4000,
                        sublocation_uid: sublocation.uid,
                    },
                };
        
                post('/api/v3/anchors', body)
                    .expect(200)
                    .then((response) => {
                        Object.assign(anchor, response.body);
                        assert.strictEqual(response.body.name, body.name, `Wrong anchor "name" value`);
                        assert.strictEqual(response.body.bridge, body.bridge, `Wrong anchor "bridge" value`);
                        assert.strictEqual(response.body.node_mac, null, `Wrong anchor "node_mac" value`);
                        assert.deepStrictEqual(response.body.position, body.position, `Wrong anchor "position" value`);
                        assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                        assert(isISOTimeFormat(response.body.updated_at), `Wrong anchor "updated_at" format`);
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should return list of anchors', checkEntities('anchors', [anchor]));
            it('should return specific anchor', function(done) {
                get(`/api/v3/anchors/${anchor.uid}`)
                    .expect(200)
                    .then((response) => {
                        assert(response.body.anchors.length === 1, `Wrong anchors number`);
                        assert.deepStrictEqual(response.body.anchors[0], anchor, `Wrong anchor object`);
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
        describe('Negative cases', function() {
            it('should fail create anchor without "position"', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: false,
                };
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"position":"REQUIRED"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor with empty position', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: true,
                    position: {},
                };
        
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `{"position":{"x":"REQUIRED","y":"REQUIRED","z":"REQUIRED","sublocation_uid":"REQUIRED"}}`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor with null position', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: true,
                    position: null,
                };
        
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, `{"position":"REQUIRED"}`, `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor without "name"', function(done) {
                const body = {
                    bridge: false,
                    position: {
                        x: -1000,
                        y: 0,
                        z: 4000,
                        sublocation_uid: sublocation.uid,
                    },
                };
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"name":"REQUIRED"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor with empty "name"', function(done) {
                const body = {
                    name: '',
                    bridge: false,
                    position: {
                        x: -1000,
                        y: 0,
                        z: 4000,
                        sublocation_uid: sublocation.uid,
                    },
                };
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"name":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor without "bridge"', function(done) {
                const body = {
                    name: 'Anchor 221',
                    position: {
                        x: -1000,
                        y: 0,
                        z: 4000,
                        sublocation_uid: sublocation.uid,
                    },
                };
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"bridge":"REQUIRED"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor with number "bridge"', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: 1,
                    position: {
                        x: -1000,
                        y: 0,
                        z: 4000,
                        sublocation_uid: sublocation.uid,
                    },
                };
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"bridge":"NOT_BOOLEAN"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor with string "bridge"', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: 'true',
                    position: {
                        x: -1000,
                        y: 0,
                        z: 4000,
                        sublocation_uid: sublocation.uid,
                    },
                };
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"bridge":"NOT_BOOLEAN"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor with invalid position', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: false,
                    position: {
                        y: 0,
                        z: 4000,
                        sublocation_uid: sublocation.uid,
                    },
                };
        
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"position":{"x":"REQUIRED"}}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor without sublocation_uid', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: true,
                    position: {
                        x: -1000,
                        y: 0,
                        z: 4000,
                    },
                };
        
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"position":{"sublocation_uid":"REQUIRED"}}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor with empty sublocation_uid', function(done) {
                const body = {
                    name: 'Anchor 221',
                    bridge: true,
                    position: {
                        x: -1000,
                        y: 0,
                        z: 4000,
                        sublocation_uid: '',
                    },
                };
        
                post('/api/v3/anchors', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"position":{"sublocation_uid":"CANNOT_BE_EMPTY"}}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create anchor with unknown sublocation uid', function(done) {
                const randomUid = UID();
                const body = {
                    name: 'Anchor 221',
                    bridge: true,
                    position: {
                        x: -1000,
                        y: 0,
                        z: 4000,
                        sublocation_uid: randomUid,
                    },
                };
        
                post('/api/v3/anchors', body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Sublocation with uid ${randomUid} not found`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
    });

    describe('Update Anchors', function () {
        it('should return NotFound error on non existed uid', function (done) {
            const uid = UID();
            const body = {
                name: 'Anchor 221',
                bridge: false,
                position: {
                    x: 1000,
                    y: 2000,
                    z: 4000,
                    sublocation_uid: sublocation.uid,
                },
            };
            put(`/api/v3/anchors/${uid}`, body)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Anchor with uid ${uid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail update anchor with invalid position', function (done) {
            const body = {
                name: 'Anchor 221',
                bridge: 'sdasdasds',
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"bridge":"NOT_BOOLEAN"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should update anchor position', function (done) {
            const body = {
                position: {
                    x: 999,
                    y: 2999,
                    z: 4999,
                    sublocation_uid: sublocation.uid,
                },
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.name, anchor.name, `Wrong anchor "name" value`);
                    assert.strictEqual(response.body.bridge, anchor.bridge, `Wrong anchor "bridge" value`);
                    assert.strictEqual(response.body.node_mac, anchor.node_mac, `Wrong anchor "node_mac" value`);
                    assert.deepStrictEqual(response.body.position, body.position, `Wrong anchor "position" value`);
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong anchor "updated_at" format`);
                    assert(
                        new Date(response.body.updated_at) > new Date(anchor.updated_at),
                        `"updated_at" should be changed`
                    );
                    assert.strictEqual(response.body.created_at, anchor.created_at, `Wrong anchor "bridge" value`);
                    anchor.position = body.position;
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail to update anchor with empty position', function (done) {
            const body = {
                position: {},
            };

            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        `{"position":{"x":"REQUIRED","y":"REQUIRED","z":"REQUIRED","sublocation_uid":"REQUIRED"}}`,
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail update anchor position with not existed sublocation', function (done) {
            const randomUid = UID();
            const body = {
                position: {
                    x: 999,
                    y: 2999,
                    z: 4999,
                    sublocation_uid: randomUid,
                },
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        `Sublocation with uid ${randomUid} not found`,
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail update anchor position without sublocation', function (done) {
            const body = {
                position: {
                    x: 999,
                    y: 2999,
                    z: 4999,
                },
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        '{"position":{"sublocation_uid":"REQUIRED"}}',
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail update anchor with empty sublocation_uid', function (done) {
            const body = {
                name: 'Anchor 221',
                bridge: true,
                position: {
                    x: -1000,
                    y: 0,
                    z: 4000,
                    sublocation_uid: '',
                },
            };

            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        '{"position":{"sublocation_uid":"CANNOT_BE_EMPTY"}}',
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail update anchor position without X', function (done) {
            const body = {
                position: {
                    y: 2999,
                    z: 4999,
                    sublocation_uid: sublocation.uid,
                },
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"position":{"x":"REQUIRED"}}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail update anchor with empty name', function (done) {
            const body = {
                name: '',
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `{"name":"CANNOT_BE_EMPTY"}`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail update anchor with empty bridge prop', function (done) {
            const body = {
                name: 'Some name',
                bridge: '',
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `{"bridge":"CANNOT_BE_EMPTY"}`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should update anchor position for another sublocation', function (done) {
            const body = {
                position: {
                    x: 999,
                    y: 2999,
                    z: 4999,
                    sublocation_uid: sublocationTwo.uid,
                },
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.name, anchor.name, `Wrong anchor "name" value`);
                    assert.strictEqual(response.body.bridge, anchor.bridge, `Wrong anchor "bridge" value`);
                    assert.strictEqual(response.body.node_mac, anchor.node_mac, `Wrong anchor "node_mac" value`);
                    assert.deepStrictEqual(response.body.position, body.position, `Wrong anchor "position" value`);
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong anchor "updated_at" format`);
                    assert(
                        new Date(response.body.updated_at) > new Date(anchor.updated_at),
                        `"updated_at" should be changed`
                    );
                    assert.strictEqual(response.body.created_at, anchor.created_at, `Wrong anchor "bridge" value`);
                    anchor.position = body.position;
                    done();
                })
                .catch((err) => done(err));
        });
        it('should update anchor name', function (done) {
            const body = {
                name: 'New Anchor Name',
            };
            put(`/api/v3/anchors/${anchor.uid}`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.name, body.name, `Wrong anchor "name" value`);
                    assert.strictEqual(response.body.bridge, anchor.bridge, `Wrong anchor "bridge" value`);
                    assert.strictEqual(response.body.node_mac, anchor.node_mac, `Wrong anchor "node_mac" value`);
                    assert.deepStrictEqual(response.body.position, anchor.position, `Wrong anchor "position" value`);
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong anchor "updated_at" format`);
                    assert(
                        new Date(response.body.updated_at) > new Date(anchor.updated_at),
                        `"updated_at" should be changed`
                    );
                    assert.strictEqual(response.body.created_at, anchor.created_at, `Wrong anchor "bridge" value`);
                    anchor.name = body.name;
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Delete Anchors', function () {
        it('should return Validation error on empty uid', function (done) {
            const uid = '';
            del(`/api/v3/anchors/${uid}`)
                .expect(404)
                .then(() => {
                    done();
                })
                .catch((err) => done(err));
        });
        it('should return NotFound error on non existed uid', function (done) {
            const uid = UID();
            del(`/api/v3/anchors/${uid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Anchor with uid ${uid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should successfully delete anchor', function (done) {
            del(`/api/v3/anchors/${anchor.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, anchor.uid, `Wrong anchor "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should return empty list of anchors', checkEntities('anchors', []));
    });

    describe('Cascade Removing', function () {
        const anchorTwo = {};
        const anchorThree = {};
        const node = {};
        const nodeAssociation = {};

        before(async function () {
            Object.assign(anchorTwo, await createAnchor(`Anchor ${UID()}`, sublocation));
            Object.assign(anchorThree, await createAnchor(`Anchor ${UID()}`, sublocation));
            Object.assign(node, await createNode('e4956ea5010b'));
            Object.assign(nodeAssociation, await createNodeAssociation(anchorTwo, node));
        });

        after(async function () {
            await del(`/api/v3/nodes/${node.mac}`);
        });

        it('should get node associations list', checkEntities('node_assignments', [nodeAssociation]));
        it('should close node association on anchor remove', function (done) {
            del(`/api/v3/anchors/${anchorTwo.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, anchorTwo.uid, `Wrong anchor "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should get node association empty list', checkEntities('node_assignments', []));
        it('should create node association with released node ', function (done) {
            post('/api/v3/node_assignments', { mac: node.mac, anchor_uid: anchorThree.uid })
                .then((response) => {
                    Object.assign(nodeAssociation, response.body);
                    assert.strictEqual(response.body.mac, node.mac, `Wrong node association "mac" value`);
                    assert.strictEqual(
                        response.body.anchor_uid,
                        anchorThree.uid,
                        `Wrong node association "anchor_uid" value`
                    );
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should close second node association on anchor remove', function (done) {
            del(`/api/v3/anchors/${anchorThree.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, anchorThree.uid, `Wrong anchor "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should get node association empty list after removal third anchor', checkEntities('node_assignments', []));
    });

    describe('ZMQ Messaging for Anchor CRUD', function () {
        const zmqObserver = new ZmqObserver();
        const anchor = {};

        before(function () {
            zmqObserver.init();
        });
        after(function () {
            zmqObserver.destroy();
        });

        it('should send zmq created anchor message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.CREATED, object: ZMQ_OBJECTS.ANCHOR };
            const body = {
                name: 'Anchor 221',
                bridge: false,
                position: {
                    x: 1000,
                    y: 2000,
                    z: 4000,
                    sublocation_uid: sublocation.uid,
                },
            };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.uid, 'ZMQ message has no uid');
                assert(zmqMsg.data.name === body.name, 'ZMQ message has wrong anchor name');
                assert(zmqMsg.data.bridge === body.bridge, 'ZMQ message has wrong anchor bridge prop');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                assert(isISOTimeFormat(zmqMsg.data.updated_at), 'Wrong ZMQ message updated_at value');
                assert(zmqMsg.data.node_mac === null, 'Wrong ZMQ message node_mac value');
                assert.deepStrictEqual(zmqMsg.data.position, body.position, 'Wrong ZMQ message position');
                done();
            });
            subscription
                .then(() => post(`/api/v3/anchors`, body))
                .then((response) => {
                    Object.assign(anchor, response.body);
                });
        });

        it('should send zmq updated anchor message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.UPDATED, object: ZMQ_OBJECTS.ANCHOR };
            const body = {
                name: 'Anchor new name',
                bridge: true,
            };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.uid === anchor.uid, 'ZMQ message has no uid');
                assert(zmqMsg.data.name === body.name, 'ZMQ message has wrong anchor name');
                assert(zmqMsg.data.bridge === body.bridge, 'ZMQ message has wrong anchor bridge prop');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                assert(isISOTimeFormat(zmqMsg.data.updated_at), 'Wrong ZMQ message updated_at value');
                assert(zmqMsg.data.node_mac === null, 'Wrong ZMQ message node_mac value');
                assert.deepStrictEqual(zmqMsg.data.position, anchor.position, 'Wrong ZMQ message position');
                done();
            });
            subscription.then(() => put(`/api/v3/anchors/${anchor.uid}`, body));
        });
        it('should send zmq deleted anchor message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.DELETED, object: ZMQ_OBJECTS.ANCHOR };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert.deepStrictEqual(zmqMsg.data, { uid: anchor.uid }, 'ZMQ message data wrong');
                done();
            });
            subscription.then(() => del(`/api/v3/anchors/${anchor.uid}`));
        });
    });
});
