import assert from 'assert';
import { UID } from '../../lib/utils/index.mjs';
import { ERROR_CODE, ZMQ_ACTIONS, ZMQ_CHANNELS, ZMQ_OBJECTS } from '../../lib/system/constants.mjs';
import { isISOTimeFormat, ZmqObserver, get, post, put, del } from '../utils/index.mjs';
import {
    createAnchor,
    createLocation,
    createNode,
    createNodeAssociation,
    createSublocation,
    checkEntities,
    Server,
} from './helpers.mjs';

const networkIdLimit = 9;
const locationsList = [];

describe('Locations Management API', function () {
    before(async function () {
        await Server().init();
    })

    after(async function () {
        for (const location of locationsList) {
            await del(`/api/v3/locations/${location.uid}`);
        }
    });

    describe('Get Empty Locations', function () {
        it('should return empty list of locations', function (done) {
            get('/api/v3/locations')
                .expect(200)
                .then((response) => {
                    assert.deepStrictEqual(response.body, { locations: [] }, `Not equal to: { locations: [] }`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should return NotFound error on wrong uid', function (done) {
            const invalidUid = UID();
            get(`/api/v3/locations/${invalidUid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Location with uid ${invalidUid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('NetworkId Management', function () {
        for (let i = 0; i < networkIdLimit; i += 1) {
            it(`should create location with network ID: ${i}  `, function (done) {
                const body = {
                    name: `Location Name ${i}`,
                };

                post('/api/v3/locations', body)
                    .expect(200)
                    .then((response) => {
                        location = response.body;
                        locationsList.push(location);
                        assert.strictEqual(response.body.name, body.name, `Wrong location "name" value`);
                        assert(typeof response.body.network_id === 'number', `Wrong location "network_id" format`);
                        assert(response.body.uid.length === 22, `Wrong location "uid" format`);
                        assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                        assert(isISOTimeFormat(response.body.updated_at), `Wrong anchor "updated_at" format`);
                        done();
                    })
                    .catch((err) => done(err));
            });
        }

        it('should free id on location removal ', function (done) {
            const location = locationsList.shift();
            del(`/api/v3/locations/${location.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, location.uid, `Wrong location "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should create location after remove ', function (done) {
            const body = {
                name: `New Location Name ${UID()}`,
            };

            post('/api/v3/locations', body)
                .expect(200)
                .then((response) => {
                    location = response.body;
                    locationsList.push(location);

                    assert.strictEqual(response.body.name, body.name, `Wrong location "name" value`);
                    assert(typeof response.body.network_id === 'number', `Wrong location "network_id" format`);
                    assert(response.body.uid.length === 22, `Wrong location "uid" format`);
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong anchor "updated_at" format`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail create location because of network_id limit exceeded', function (done) {
            const body = {
                name: `New Location Name ${UID()}`,
            };

            post('/api/v3/locations', body)
                .expect(500)
                .then((response) => {
                    assert(response.body.message, 'No free network_id');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should delete location', function (done) {
            const location = locationsList.shift();
            del(`/api/v3/locations/${location.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, location.uid, `Wrong location "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });
    });

    let location = null;
    describe('Create Locations', function () {
        it('should create location', function (done) {
            const body = {
                name: 'Location Name',
            };

            post('/api/v3/locations', body)
                .expect(200)
                .then((response) => {
                    location = response.body;
                    assert.strictEqual(response.body.name, body.name, `Wrong location "name" value`);
                    assert(typeof response.body.network_id === 'number', `Wrong location "network_id" format`);
                    assert(response.body.uid.length === 22, `Wrong location "uid" format`);
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong anchor "updated_at" format`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail create location without "name"', function (done) {
            const body = {};
            post('/api/v3/locations', body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"name":"REQUIRED"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail create location with empty "name"', function (done) {
            const body = {
                name: '',
            };
            post('/api/v3/locations', body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"name":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should return list of locations', function (done) {
            get('/api/v3/locations')
                .expect(200)
                .then((response) => {
                    assert(response.body.locations.length === 9, `Wrong locations number`);
                    // assert.deepStrictEqual(response.body.locations[0], location, `Wrong location object`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should return specific location', function (done) {
            get(`/api/v3/locations/${location.uid}`)
                .expect(200)
                .then((response) => {
                    assert(response.body.locations.length === 1, `Wrong locations number`);
                    assert.deepStrictEqual(response.body.locations[0], location, `Wrong locations object`);
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Update Locations', function () {
        it('should return NotFound error on non existed uid', function (done) {
            const uid = UID();
            const body = {
                name: 'New Location Name',
            };
            put(`/api/v3/locations/${uid}`, body)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Location with uid ${uid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail update location with empty "name"', function (done) {
            const body = {
                name: '',
            };
            put(`/api/v3/locations/${location.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"name":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should update location name', function (done) {
            const body = {
                name: 'New Location Name',
            };
            put(`/api/v3/locations/${location.uid}`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.name, body.name, `Wrong location "name" value`);
                    assert.strictEqual(
                        response.body.network_id,
                        location.network_id,
                        `Wrong location "network_id" value`
                    );
                    assert(isISOTimeFormat(response.body.created_at), `Wrong location "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong location "updated_at" format`);
                    assert(
                        new Date(response.body.updated_at) > new Date(location.updated_at),
                        `"updated_at" should be changed`
                    );
                    assert.strictEqual(
                        response.body.created_at,
                        location.created_at,
                        `created_at should not be changed`
                    );
                    location.name = body.name;
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Delete Locations', function () {
        it('should return NotFound error on non existed uid', function (done) {
            const uid = UID();
            del(`/api/v3/locations/${uid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Location with uid ${uid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should successfully delete location', function (done) {
            del(`/api/v3/locations/${location.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, location.uid, `Wrong location "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should not found removed location', function (done) {
            get(`/api/v3/locations/${location.uid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        `Location with uid ${location.uid} not found`,
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Cascade Removal', function () {
        const node = {};
        const anchor = {};
        const nodeAssociation = {};
        const sublocation = {};
        const locationTwo = {};

        before(async function () {
            Object.assign(locationTwo, await createLocation(`Location ${UID()}`));
            Object.assign(sublocation, await createSublocation(`Sublocation ${UID()}`, locationTwo));
            Object.assign(anchor, await createAnchor(`Anchor ${UID()}`, sublocation));
            Object.assign(node, await createNode('e4956ea5010b'));
            Object.assign(nodeAssociation, await createNodeAssociation(anchor, node));
            anchor.node_mac = node.mac;
        });

        after(async function () {
            await del(`/api/v3/nodes/${node.mac}`);
        });

        it('should return list of sublocations', checkEntities('sublocations', [sublocation]));

        it('should return list of node associations', checkEntities('node_assignments', [nodeAssociation]));

        it('should return list of anchors', checkEntities('anchors', [anchor]));

        it('should remove location', function (done) {
            del(`/api/v3/locations/${locationTwo.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, locationTwo.uid, `Wrong location "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should return empty list of node associations', checkEntities('node_assignments', []));

        it('should return empty list of anchors', checkEntities('anchors', []));

        it('should return empty list of sublocations', checkEntities('sublocations', []));
    });

    describe.skip('ZMQ Messaging for Location CRUD', function () {
        const zmqObserver = new ZmqObserver();
        const locationZmq = {};

        before(function () {
            zmqObserver.init();
        });
        after(function () {
            zmqObserver.destroy();
        });

        it('should send zmq created location message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.CREATED, object: ZMQ_OBJECTS.LOCATION };
            const body = {
                name: 'Location Zmq Test',
            };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.uid, 'ZMQ message has no uid');
                assert(zmqMsg.data.name === body.name, 'ZMQ message has wrong location name');
                assert(zmqMsg.data.network_id, 'ZMQ message has wrong network_id prop');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                assert(isISOTimeFormat(zmqMsg.data.updated_at), 'Wrong ZMQ message updated_at value');
                done();
            });
            subscription
                .then(() => post(`/api/v3/locations`, body))
                .then((response) => {
                    Object.assign(locationZmq, response.body);
                });
        });

        it('should send zmq updated location message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.UPDATED, object: ZMQ_OBJECTS.LOCATION };
            const body = {
                name: 'Location Zmq New Name',
            };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.uid === locationZmq.uid, 'ZMQ message has wrong uid');
                assert(zmqMsg.data.name === body.name, 'ZMQ message has wrong location name');
                assert(zmqMsg.data.network_id, 'ZMQ message has wrong network_id prop');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                assert(isISOTimeFormat(zmqMsg.data.updated_at), 'Wrong ZMQ message updated_at value');
                done();
            });
            subscription.then(() => put(`/api/v3/locations/${locationZmq.uid}`, body));
        });
        it('should send zmq deleted location message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.DELETED, object: ZMQ_OBJECTS.LOCATION };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert.deepStrictEqual(zmqMsg.data, { uid: locationZmq.uid }, 'ZMQ message data wrong');
                done();
            });
            subscription.then(() => del(`/api/v3/locations/${locationZmq.uid}`));
        });
    });
});
