import assert from 'assert';
import { UID } from '../../lib/utils/index.mjs';
import { ERROR_CODE } from '../../lib/system/constants.mjs';
import { isISOTimeFormat, get, post, put, del } from '../utils/index.mjs';
import { createLocation, checkEntities, Server } from './helpers.mjs';

const location1 = {};
const location2 = {};
const schedule1 = {};
const schedule2 = {};

describe('Network LSB schedule Management API', function() {
    before(async function() {
        await Server().init();
        Object.assign(location1, await createLocation('Location Name1'));
        Object.assign(location2, await createLocation('Location Name2'));
    });

    after(async function() {
        await del(`/api/v3/locations/${location1.uid}`);
        await del(`/api/v3/locations/${location2.uid}`);
    });

    describe('Get Empty LSB schedule', function() {
        it('should return empty list of sublocations', checkEntities('network/lsb_schedules', [], 'lsb_schedules'));

        it('should return NotFound error on wrong uid', function(done) {
            const invalidUid = UID();
            get(`/api/v3/network/lsb_schedules/${invalidUid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        `Network schedule with uid ${invalidUid} not found`,
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Create LSB schedule', function() {
        describe('Negative cases', function() {
            it('should fail create LSB schedule without location_uid', function(done) {
                const body = {
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"location_uid":"REQUIRED"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with wrong type location_uid', function(done) {
                const body = {
                    name: 'name',
                    location_uid: 1,
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"location_uid":"NOT_STRING"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with empty location_uid', function(done) {
                const body = {
                    location_uid: '',
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"location_uid":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with wrong location_uid', function(done) {
                const fakeUid = '123qweasdzxc123qweasdz';
                const body = {
                    location_uid: fakeUid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Location with uid ${fakeUid} not found`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without name', function(done) {
                const body = {
                    location_uid: location1.uid,
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"name":"REQUIRED"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with empty name', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: '',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"name":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without content', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":"REQUIRED"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with empty content', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [],
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without content.source', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"source":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with empty content.source', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: '',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"source":"CANNOT_BE_EMPTY"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without content.destination', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"destination":"REQUIRED"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with empty content.destination', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: '',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"destination":"CANNOT_BE_EMPTY"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without content.section', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"section":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with wrong number content.section', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 3,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"section":"NOT_ALLOWED_VALUE"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with wrong string content.section', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: '1',
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"section":"NOT_INTEGER"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without content.boundary', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"boundary":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without content.slot', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"slot":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with min_number content.slot', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: -1,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"slot":"TOO_LOW"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with max_number content.slot', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 255,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"slot":"TOO_HIGH"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without content.margin', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"margin":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with min_number content.margin', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: -250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"margin":"TOO_LOW"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with max_number content.margin', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 50250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"margin":"TOO_HIGH"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with multiple content.margin', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 253,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"margin":"NOT_MULTIPLE_TO 250"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule without content.dimension', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"dimension":"REQUIRED"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with empty content.dimension', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"dimension":"CANNOT_BE_EMPTY"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with wrong content.dimension', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '4D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"dimension":"NOT_ALLOWED_VALUE"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create LSB schedule with wrong content.section and content.dimension', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '2D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            'For content.dimension 2D available only content.section 0',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
        
        describe('Positive cases', function() {
            it('should create LSB schedule #1', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'LSB schedule #1',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(200)
                    .then((response) => {
                        assert(response.body.uid.length === 22, `Wrong sublocation "uid" format`);
                        assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                        assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);

                        assert.strictEqual(response.body.name, body.name, `Wrong sublocation "name" value`);
                        assert.strictEqual(response.body.location_uid, body.location_uid, `Wrong "location_uid" value`);
                        assert.strictEqual(
                            response.body.content[0].source,
                            body.content[0].source,
                            `Wrong "content[0].source" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].destination,
                            body.content[0].destination,
                            `Wrong "content[0].destination" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].slot,
                            body.content[0].slot,
                            `Wrong "content[0].slot" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].section,
                            body.content[0].section,
                            `Wrong "content[0].section" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].dimension,
                            body.content[0].dimension,
                            `Wrong "content[0].dimension" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].boundary,
                            body.content[0].boundary,
                            `Wrong "content[0].boundary" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].margin,
                            body.content[0].margin,
                            `Wrong "content[0].margin" value`
                        );
                        Object.assign(schedule1, response.body);

                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should create LSB schedule #2', function(done) {
                const body = {
                    location_uid: location2.uid,
                    name: 'LSB schedule #2',
                    content: [{
                        source: 'sourceAnchorUid2wewwww',
                        destination: 'destinationAnchorUid22',
                        slot: 0,
                        section: 1,
                        dimension: '0d',
                        boundary: true,
                        margin: 250,
                    }]
                };
                post('/api/v3/network/lsb_schedules', body)
                    .expect(200)
                    .then((response) => {
                        assert(response.body.uid.length === 22, `Wrong sublocation "uid" format`);
                        assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                        assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);

                        assert.strictEqual(response.body.name, body.name, `Wrong sublocation "name" value`);
                        assert.strictEqual(response.body.location_uid, body.location_uid, `Wrong "location_uid" value`);
                        assert.strictEqual(
                            response.body.content[0].source,
                            body.content[0].source,
                            `Wrong "content[0].source" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].destination,
                            body.content[0].destination,
                            `Wrong "content[0].destination" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].slot,
                            body.content[0].slot,
                            `Wrong "content[0].slot" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].section,
                            body.content[0].section,
                            `Wrong "content[0].section" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].dimension,
                            body.content[0].dimension.toUpperCase(),
                            `Wrong "content[0].dimension" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].boundary,
                            body.content[0].boundary,
                            `Wrong "content[0].boundary" value`
                        );
                        assert.strictEqual(
                            response.body.content[0].margin,
                            body.content[0].margin,
                            `Wrong "content[0].margin" value`
                        );
                        Object.assign(schedule2, response.body);

                        done();
                    })
                    .catch((err) => done(err));
            });
        });
    });

    describe('Get LSB schedules', function() {
        it('should return list of all schedules 1', function() {
            checkEntities('network/lsb_schedules', [schedule1, schedule2], 'lsb_schedules')();
        });

        it('should return list schedules by location 1', function() {
            checkEntities(`network/lsb_schedules?location_uid=${location1.uid}`, [schedule1], 'lsb_schedules')();
        });

        it('should return list schedules by location 2', function() {
            checkEntities(`network/lsb_schedules?location_uid=${location2.uid}`, [schedule2], 'lsb_schedules')();
        });
    });

    describe('Update LSB schedule', function() {
        describe('Negative cases', function() {
            it('should fail update LSB schedule with empty name', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: '',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"name":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with empty content', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [],
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule without content.source', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"source":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with empty content.source', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: '',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"source":"CANNOT_BE_EMPTY"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule without content.destination', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"destination":"REQUIRED"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with empty content.destination', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: '',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"destination":"CANNOT_BE_EMPTY"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule without content.section', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"section":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule without content.boundary', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"boundary":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule without content.slot', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"slot":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with min_number content.slot', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: -1,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"slot":"TOO_LOW"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with max_number content.slot', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 255,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"slot":"TOO_HIGH"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule without content.margin', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"margin":"REQUIRED"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with min_number content.margin', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: -250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"margin":"TOO_LOW"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with max_number content.margin', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 50250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"content":[{"margin":"TOO_HIGH"}]}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with multiple content.margin', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '0D',
                        boundary: true,
                        margin: 253,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"margin":"NOT_MULTIPLE_TO 250"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule without content.dimension', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"dimension":"REQUIRED"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with empty content.dimension', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"dimension":"CANNOT_BE_EMPTY"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with wrong content.dimension', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '4D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"content":[{"dimension":"NOT_ALLOWED_VALUE"}]}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail update LSB schedule with wrong content.section and content.dimension', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '2D',
                        boundary: true,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            'For content.dimension 2D available only content.section 0',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
        });

        describe('Positive cases', function() {
            it('should update LSB schedule without content', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(200)
                    .then((response) => {
                        assert(response.body.uid.length === 22, `Wrong sublocation "uid" format`);
                        assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                        assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);
                        assert.deepStrictEqual(response.body.content, schedule1.content, `Wrong content`);
                        assert.strictEqual(response.body.name, body.name, `Wrong content`);
                
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should update LSB schedule', function(done) {
                const body = {
                    location_uid: location1.uid,
                    name: 'name',
                    content: [{
                        source: 'sourceAnchorUid1wewwww',
                        destination: 'destinationAnchorUid11',
                        slot: 0,
                        section: 1,
                        dimension: '1D',
                        boundary: false,
                        margin: 250,
                    }]
                };
                put(`/api/v3/network/lsb_schedules/${schedule1.uid}`, body)
                    .expect(200)
                    .then((response) => {
                        assert(response.body.uid.length === 22, `Wrong sublocation "uid" format`);
                        assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                        assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);

                        assert.strictEqual(response.body.name, body.name, `Wrong sublocation "name" value`);
                        assert.strictEqual(response.body.location_uid, body.location_uid, `Wrong "location_uid" value`);
                        assert.strictEqual(
                            response.body.content.source,
                            body.content.source,
                            `Wrong "content.source" value`
                        );
                        assert.strictEqual(
                            response.body.content.destination,
                            body.content.destination,
                            `Wrong "content.destination" value`
                        );
                        assert.strictEqual(response.body.content.slot, body.content.slot, `Wrong "content.slot" value`);
                        assert.strictEqual(
                            response.body.content.section,
                            body.content.section,
                            `Wrong "content.section" value`
                        );
                        assert.strictEqual(
                            response.body.content.dimension,
                            body.content.dimension,
                            `Wrong "content.dimension" value`
                        );
                        assert.strictEqual(
                            response.body.content.boundary,
                            body.content.boundary,
                            `Wrong "content.boundary" value`
                        );
                        assert.strictEqual(
                            response.body.content.margin,
                            body.content.margin,
                            `Wrong "content.margin" value`
                        );
                        Object.assign(schedule1, response.body);

                        done();
                    })
                    .catch((err) => done(err));
            });
        });
    });

    describe('Delete LSB schedule', function() {
        it('should return NotFound error on non existed uid', function(done) {
            const uid = UID();
            del(`/api/v3/network/lsb_schedules/${uid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        `Network schedule with uid ${uid} not found`,
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should successfully delete schedule #1', function(done) {
            del(`/api/v3/locations/${location1.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, location1.uid, `Wrong location "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should successfully delete schedule #2', function(done) {
            del(`/api/v3/network/lsb_schedules/${schedule2.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, schedule2.uid, `Wrong sublocation "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should not found removed schedule', function(done) {
            get(`/api/v3/network/lsb_schedules/${schedule1.uid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        `Network schedule with uid ${schedule1.uid} not found`,
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should return empty list of sublocations', checkEntities('network/lsb_schedules', [], 'lsb_schedules'));
    });
});
