import assert from 'assert';
import { randomMac, isUpperCaseString, isISOTimeFormat, ZmqObserver, get, post, del } from '../utils/index.mjs';
import { createDevice, checkEntities, Server } from './helpers.mjs';
import { ZMQ_ACTIONS, ZMQ_CHANNELS, ZMQ_OBJECTS } from '../../lib/system/constants.mjs';

const idLimit = 9;
const nodesList = [];
const tagsList = [];

describe('Nodes/Tags Management API', function () {
    before(async function () {
        await Server().init();
    })

    describe('node_id tests', function () {
        after(async function () {
            for (const node of nodesList) {
                await del(`/api/v3/nodes/${node.mac}`);
            }
            for (const tag of tagsList) {
                await del(`/api/v3/tags/${tag.mac}`);
            }
        });

        for (let i = 0; i < idLimit; i += 1) {
            it(`should create node with ID (${i + 1})`, function (done) {
                const body = { mac: randomMac() };
                const even = i % 2 === 0;
                post(`/api/v3/${even ? 'nodes' : 'tags'}`, body)
                    .expect(200)
                    .then((response) => {
                        if (even) nodesList.push(response.body);
                        if (!even) tagsList.push(response.body);
                        assert(response.body.node_id, '"node_id" should exists');
                        assert(response.body.node_id <= idLimit, `"node_id" should be less than ${idLimit}`);
                        assert(isUpperCaseString(response.body.mac), 'MAC address should be in upper case');
                        assert(isISOTimeFormat(response.body.created_at), '"created_at" should be in ISO 8601 format');
                        assert(response.body.last_heard === null, '"last_heard" should be equal to null on create');
                        assert(response.body.cbid === null, '"cbid" should be equal to null on create');
                        done();
                    })
                    .catch((err) => done(err));
            });
        }

        it('should fail create node because of node_id limit exceed', function (done) {
            const body = { mac: randomMac() };
            post('/api/v3/nodes', body)
                .expect(500)
                .then((response) => {
                    assert(response.body.message, 'No free node_id');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fail create tag because of node_id limit exceed', function (done) {
            const body = { mac: randomMac() };
            post('/api/v3/tags', body)
                .expect(500)
                .then((response) => {
                    assert(response.body.message, 'No free node_id');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should delete tag and make one node_id available', function (done) {
            const tag = tagsList.shift();
            del(`/api/v3/tags/${tag.mac}`)
                .expect(200)
                .then(() => {
                    done();
                })
                .catch((err) => done(err));
        });
        it('should create node with released id', function (done) {
            const body = {
                mac: randomMac(),
            };
            post('/api/v3/nodes', body)
                .expect(200)
                .then((response) => {
                    nodesList.push(response.body);
                    assert(response.body.node_id, '"node_id" should exists');
                    assert(response.body.node_id <= idLimit, `"node_id" should be less then ${idLimit}`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should delete node and make one node_id available', function (done) {
            const node = nodesList.shift();
            del(`/api/v3/nodes/${node.mac}`)
                .expect(200)
                .then(() => {
                    done();
                })
                .catch((err) => done(err));
        });
        it('should create tag with released id', function (done) {
            const body = {
                mac: randomMac(),
            };
            post('/api/v3/tags', body)
                .expect(200)
                .then((response) => {
                    tagsList.push(response.body);
                    assert(response.body.node_id, '"node_id" should exists');
                    assert(response.body.node_id <= idLimit, `"node_id" should be less then ${idLimit}`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should get all nodes list', function (done) {
            get('/api/v3/nodes')
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.nodes.length, nodesList.length, 'nodes lists should be equal');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should get all tags list', function (done) {
            get('/api/v3/tags')
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.tags.length, tagsList.length, 'tags lists should be equal');
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Nodes CRUD', function () {
        const nodeOne = {};
        const nodeTwo = {};

        it('should create node one', function (done) {
            const body = { mac: randomMac() };
            post('/api/v3/nodes', body)
                .expect(200)
                .then((response) => {
                    Object.assign(nodeOne, response.body);
                    assert(response.body.node_id, '"node_id" should exists');
                    assert(response.body.mac, '"mac" should exists');
                    assert(response.body.cbid === null, '"cbid" should be null');
                    assert(isUpperCaseString(response.body.mac), '"mac" should be upper case');
                    assert.strictEqual(
                        response.body.mac.toUpperCase(),
                        body.mac.toUpperCase(),
                        '"macs" should be equal'
                    );
                    assert(response.body.last_heard === null, '"last_heard" should be null');
                    assert(isISOTimeFormat(response.body.created_at), '"created_at" should be ISO 8601 format');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should create node two', function (done) {
            const body = { mac: randomMac() };
            post('/api/v3/nodes', body)
                .expect(200)
                .then((response) => {
                    Object.assign(nodeTwo, response.body);
                    assert(response.body.node_id, '"node_id" should exists');
                    assert(response.body.mac, '"mac" should exists');
                    assert(response.body.cbid === null, '"cbid" should be null');
                    assert(isUpperCaseString(response.body.mac), '"mac" should be upper case');
                    assert.strictEqual(
                        response.body.mac.toUpperCase(),
                        body.mac.toUpperCase(),
                        '"macs" should be equal'
                    );
                    assert(response.body.last_heard === null, '"last_heard" should be null');
                    assert(isISOTimeFormat(response.body.created_at), '"created_at" should be ISO 8601 format');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create node without MAC, REQUIRED', function (done) {
            const body = {};
            post('/api/v3/nodes', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(response.body.message, '{"mac":"REQUIRED"}', 'Mac is required');
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac is required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create node with empty MAC, CANNOT_BE_EMPTY', function (done) {
            const body = { mac: '' };
            post('/api/v3/nodes', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(response.body.message, '{"mac":"CANNOT_BE_EMPTY"}', 'Mac cannot be empty');
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create node with MAC less 12, WRONG_MAC_FORMAT', function (done) {
            const body = { mac: 'e4956ea5127' };
            post('/api/v3/nodes', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(response.body.message, '{"mac":"WRONG_MAC_FORMAT"}', 'Mac wrong mac format');
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create node with MAC more 12, WRONG_MAC_FORMAT', function (done) {
            const body = { mac: 'e4956ea512e7e' };
            post('/api/v3/nodes', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(response.body.message, '{"mac":"WRONG_MAC_FORMAT"}', 'Mac wrong mac format');
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create node with invalid MAC, WRONG_MAC_FORMAT', function (done) {
            const body = { mac: 'MAC12345IT22' };
            post('/api/v3/nodes', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(
                        response.body.message,
                        '{"mac":"WRONG_MAC_FORMAT"}',
                        'Mac wrong format required'
                    );
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should get all nodes list', function (done) {
            get('/api/v3/nodes')
                .expect(200)
                .then((response) => {
                    assert(response.body.nodes.length, 2, 'length should be 2');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should get single node', function (done) {
            get(`/api/v3/nodes/${nodeOne.mac}`)
                .expect(200)
                .then((response) => {
                    assert(response.body.nodes.length, 1, 'length should be 1');
                    assert(response.body.nodes[0].mac === nodeOne.mac.toUpperCase(), 'macs should be equal');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails delete node by wrong MAC', function (done) {
            del(`/api/v3/nodes/MAC12345IT22`)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(
                        response.body.message,
                        '{"mac":"WRONG_MAC_FORMAT"}',
                        'Mac wrong format required'
                    );
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should delete nodeOne by MAC', function (done) {
            del(`/api/v3/nodes/${nodeOne.mac.toLowerCase()}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.mac, nodeOne.mac, `Wrong node "mac" value`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should delete nodeTwo by MAC', function (done) {
            del(`/api/v3/nodes/${nodeTwo.mac.toLowerCase()}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.mac, nodeTwo.mac, `Wrong node "mac" value`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to get node by wrong MAC', function (done) {
            const mac = randomMac();
            get(`/api/v3/nodes/${mac}`)
                .expect(404)
                .then((response) => {
                    assert.strictEqual(
                        response.body.message,
                        `Node with MAC ${mac.toUpperCase()} not found`,
                        'Wrong error message'
                    );
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Tags CRUD', function () {
        const tagOne = {};
        const tagTwo = {};

        it('should create tag one', function (done) {
            const body = { mac: randomMac() };
            post('/api/v3/tags', body)
                .expect(200)
                .then((response) => {
                    Object.assign(tagOne, response.body);
                    assert(response.body.node_id, '"node_id" should exists');
                    assert(response.body.mac, '"mac" should exists');
                    assert(response.body.cbid === null, '"cbid" should be null');
                    assert(isUpperCaseString(response.body.mac), '"mac" should be upper case');
                    assert.strictEqual(
                        response.body.mac.toUpperCase(),
                        body.mac.toUpperCase(),
                        '"macs" should be equal'
                    );
                    assert(response.body.last_heard === null, '"last_heard" should be null');
                    assert(isISOTimeFormat(response.body.created_at), '"created_at" should be ISO 8601 format');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should create tag two', function (done) {
            const body = { mac: randomMac() };
            post('/api/v3/tags', body)
                .expect(200)
                .then((response) => {
                    Object.assign(tagTwo, response.body);
                    assert(response.body.node_id, '"node_id" should exists');
                    assert(response.body.mac, '"mac" should exists');
                    assert(response.body.cbid === null, '"cbid" should be null');
                    assert(isUpperCaseString(response.body.mac), '"mac" should be upper case');
                    assert.strictEqual(
                        response.body.mac.toUpperCase(),
                        body.mac.toUpperCase(),
                        '"macs" should be equal'
                    );
                    assert(response.body.last_heard === null, '"last_heard" should be null');
                    assert(isISOTimeFormat(response.body.created_at), '"created_at" should be ISO 8601 format');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create tag without MAC, REQUIRED', function (done) {
            const body = {};
            post('/api/v3/tags', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(response.body.message, '{"mac":"REQUIRED"}', 'Mac is required');
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac is required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create tag with empty MAC, CANNOT_BE_EMPTY', function (done) {
            const body = { mac: '' };
            post('/api/v3/tags', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(response.body.message, '{"mac":"CANNOT_BE_EMPTY"}', 'Mac cannot be empty');
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create tag with MAC less 12, WRONG_MAC_FORMAT', function (done) {
            const body = { mac: 'e4956ea5127' };
            post('/api/v3/tags', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(response.body.message, '{"mac":"WRONG_MAC_FORMAT"}', 'Mac wrong mac format');
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create tag with MAC more 12, WRONG_MAC_FORMAT', function (done) {
            const body = { mac: 'e4956ea512e7e' };
            post('/api/v3/tags', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(response.body.message, '{"mac":"WRONG_MAC_FORMAT"}', 'Mac wrong mac format');
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to create tag with invalid MAC 12, WRONG_MAC_FORMAT', function (done) {
            const body = { mac: 'MAC12345IT22' };
            post('/api/v3/tags', body)
                .expect(422)
                .then((response) => {
                    assert.strictEqual(
                        response.body.message,
                        '{"mac":"WRONG_MAC_FORMAT"}',
                        'Mac wrong format required'
                    );
                    assert.strictEqual(response.body.code, 'VALIDATION_ERROR', 'Mac wrong format required');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should get all tags list', function (done) {
            get('/api/v3/tags')
                .expect(200)
                .then((response) => {
                    assert(response.body.tags.length, 2, 'length should be 2');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should get single tag', function (done) {
            get(`/api/v3/tags/${tagOne.mac}`)
                .expect(200)
                .then((response) => {
                    assert(response.body.tags.length, 1, 'length should be 1');
                    assert(response.body.tags[0].mac === tagOne.mac.toUpperCase(), 'macs should be equal');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should delete tagOne by MAC', function (done) {
            del(`/api/v3/tags/${tagOne.mac.toLowerCase()}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.mac, tagOne.mac, `Wrong tag "mac" value`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should delete tagTwo by MAC', function (done) {
            del(`/api/v3/tags/${tagTwo.mac.toLowerCase()}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.mac, tagTwo.mac, `Wrong tag "mac" value`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should fails to get tag by wrong MAC', function (done) {
            const mac = randomMac();
            get(`/api/v3/tags/${mac}`)
                .expect(404)
                .then((response) => {
                    assert.strictEqual(
                        response.body.message,
                        `Tag with MAC ${mac.toUpperCase()} not found`,
                        'Wrong error message'
                    );
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Devices CRUD', function () {
        const deviceOne = {};
        const deviceTwo = {};

        before(async function () {
            Object.assign(deviceOne, await createDevice(randomMac()));
            Object.assign(deviceTwo, await createDevice(randomMac()));
        });
        after(async function () {
            await del(`/api/v3/nodes/${deviceOne.mac}`);
            await del(`/api/v3/tags/${deviceTwo.mac}`);
        });

        it('should get all hardware_devices', function (done) {
            get('/api/v3/hardware_devices')
                .expect(200)
                .then((response) => {
                    assert(response.body.hardware_devices.length, 2, 'length should be 2');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should return empty list of nodes', checkEntities('nodes', []));
        it('should return empty list of tags', checkEntities('tags', []));
        it('should make node from device', function (done) {
            const body = { mac: deviceOne.mac };
            post('/api/v3/nodes', body)
                .expect(200)
                .then((response) => {
                    assert(response.body.node_id, '"node_id" should exists');
                    assert(response.body.node_id === deviceOne.node_id, '"node_id" should no be changed');
                    assert(isISOTimeFormat(response.body.created_at), '"created_at" should be ISO 8601 format');
                    assert(response.body.created_at !== deviceOne.created_at, '"created_at" should be changed');
                    Object.assign(deviceOne, response.body);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should make tag from device', function (done) {
            const body = { mac: deviceTwo.mac };
            post('/api/v3/tags', body)
                .expect(200)
                .then((response) => {
                    assert(response.body.node_id, '"node_id" should exists');
                    assert(response.body.node_id === deviceTwo.node_id, '"node_id" should no be changed');
                    assert(isISOTimeFormat(response.body.created_at), '"created_at" should be ISO 8601 format');
                    assert(response.body.created_at !== deviceTwo.created_at, '"created_at" should be changed');
                    Object.assign(deviceTwo, response.body);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should return empty list of devices', checkEntities('hardware_devices', []));
        it('should return list of nodes', checkEntities('nodes', [deviceOne]));
        it('should return list of tags', checkEntities('tags', [deviceTwo]));
    });

    describe('ZMQ Messaging for Nodes/Tags CRUD', function () {
        const zmqObserver = new ZmqObserver();
        const nodeOne = {};
        const tagOne = {};

        before(function () {
            zmqObserver.init();
        });
        after(function () {
            zmqObserver.destroy();
        });

        it('should send zmq created node message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.CREATED, object: ZMQ_OBJECTS.NODE };
            const body = { mac: randomMac() };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.node_id, 'ZMQ message has no node_id');
                assert(zmqMsg.data.mac === body.mac.toUpperCase(), 'ZMQ message has wrong node MAC');
                assert(zmqMsg.data.last_heard === null, 'ZMQ message has wrong node "last_heard" prop');
                assert(zmqMsg.data.cbid === null, 'ZMQ message has wrong node "cbid" prop');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                done();
            });
            subscription
                .then(() => post(`/api/v3/nodes`, body))
                .then((response) => {
                    Object.assign(nodeOne, response.body);
                });
        });
        it('should send zmq deleted node message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.DELETED, object: ZMQ_OBJECTS.NODE };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert.deepStrictEqual(zmqMsg.data, { mac: nodeOne.mac.toUpperCase() }, 'ZMQ message data wrong');
                done();
            });
            subscription.then(() => del(`/api/v3/nodes/${nodeOne.mac}`));
        });
        it('should send zmq created tag message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.CREATED, object: ZMQ_OBJECTS.TAG };
            const body = { mac: randomMac() };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.node_id, 'ZMQ message has no node_id');
                assert(zmqMsg.data.mac === body.mac.toUpperCase(), 'ZMQ message has wrong tag MAC');
                assert(zmqMsg.data.last_heard === null, 'ZMQ message has wrong tag "last_heard" prop');
                assert(zmqMsg.data.cbid === null, 'ZMQ message has wrong tag "cbid" prop');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                done();
            });
            subscription
                .then(() => post(`/api/v3/tags`, body))
                .then((response) => {
                    Object.assign(tagOne, response.body);
                });
        });
        it('should send zmq deleted tag message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.DELETED, object: ZMQ_OBJECTS.TAG };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert.deepStrictEqual(zmqMsg.data, { mac: tagOne.mac.toUpperCase() }, 'ZMQ message data wrong');
                done();
            });
            subscription.then(() => del(`/api/v3/tags/${tagOne.mac}`));
        });
    });
});
