import assert from 'assert';
import { UID } from '../../lib/utils/index.mjs';
import { ERROR_CODE, ZMQ_ACTIONS, ZMQ_CHANNELS, ZMQ_OBJECTS } from '../../lib/system/constants.mjs';
import { isISOTimeFormat, ZmqObserver, get, post, put, del } from '../utils/index.mjs';
import {
    createLocation,
    createAnchor,
    createSublocation,
    createNodeAssociation,
    createNode,
    checkEntities,
    Server,
} from './helpers.mjs';

const sublocationIdLimit = 9;
const sublocationsList = [];
const location = {};

describe('Sublocations Management API', function () {
    before(async function () {
        await Server().init();
        Object.assign(location, await createLocation('Location Name'));
    });

    after(async function () {
        await del(`/api/v3/locations/${location.uid}`);
        for (const sublocation of sublocationsList) {
            await del(`/api/v3/sublocations/${sublocation.uid}`);
        }
    });

    describe('Get Empty Sublocations', function () {
        it('should return empty list of sublocations', checkEntities('sublocations', []));

        it('should return NotFound error on wrong uid', function (done) {
            const invalidUid = UID();
            get(`/api/v3/sublocations/${invalidUid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        `Sublocation with uid ${invalidUid} not found`,
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('SublocationId Management', function () {
        for (let i = 0; i < sublocationIdLimit; i += 1) {
            it('should create sublocation with ID', function (done) {
                const body = { name: `Sublocation Name ${i}`, location_uid: location.uid };

                post('/api/v3/sublocations', body)
                    .expect(200)
                    .then((response) => {
                        sublocationsList.push(response.body);
                        assert.strictEqual(response.body.name, body.name, `Wrong sublocation "name" value`);
                        assert(typeof response.body.id === 'number', `Wrong sublocation "id" format`);
                        assert(response.body.uid.length === 22, `Wrong sublocation "uid" format`);
                        assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                        assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);
                        done();
                    })
                    .catch((err) => done(err));
            });
        }

        it('should free id on sublocation removal ', function (done) {
            const sublocation = sublocationsList.shift();
            del(`/api/v3/sublocations/${sublocation.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, sublocation.uid, `Wrong sublocation "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should create sublocation after remove ', function (done) {
            const body = {
                name: `New Sublocation Name ${UID()}`,
                location_uid: location.uid,
            };

            post('/api/v3/sublocations', body)
                .expect(200)
                .then((response) => {
                    sublocationsList.push(response.body);

                    assert.strictEqual(response.body.name, body.name, `Wrong sublocation "name" value`);
                    assert(typeof response.body.id === 'number', `Wrong sublocation "id" format`);
                    assert(response.body.uid.length === 22, `Wrong sublocation "uid" format`);
                    assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail create sublocation because of id limit exceeded', function (done) {
            const body = {
                name: `New Sublocation Name ${UID()}`,
                location_uid: location.uid,
            };

            post('/api/v3/sublocations', body)
                .expect(500)
                .then((response) => {
                    assert(response.body.message, 'No free id');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should delete sublocation', function (done) {
            const sublocation = sublocationsList.shift();
            del(`/api/v3/sublocations/${sublocation.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, sublocation.uid, `Wrong sublocation "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });
    });

    const sublocation = {};
    describe('Create Sublocations', function () {
        it('should create sublocation', function (done) {
            const body = {
                name: 'Sublocation Name',
                location_uid: location.uid,
            };

            post('/api/v3/sublocations', body)
                .expect(200)
                .then((response) => {
                    Object.assign(sublocation, response.body);
                    assert.strictEqual(response.body.name, body.name, `Wrong sublocation "name" value`);
                    assert(typeof response.body.id === 'number', `Wrong sublocation "id" format`);
                    assert(response.body.uid.length === 22, `Wrong sublocation "uid" format`);
                    assert(typeof response.body.elevation === 'number', `Wrong sublocation "elevation" format`);
                    assert(response.body.elevation === 0, `Wrong default elevation value`);
                    assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail create sublocation without "name"', function (done) {
            const body = { location_uid: location.uid };
            post('/api/v3/sublocations', body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"name":"REQUIRED"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail create sublocation with empty "name"', function (done) {
            const body = {
                location_uid: location.uid,
                name: '',
            };
            post('/api/v3/sublocations', body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"name":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail create sublocation without "location_uid"', function (done) {
            const body = { name: `Sublocation ${UID()}` };
            post('/api/v3/sublocations', body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"location_uid":"REQUIRED"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail create sublocation with empty "location_uid", CANNOT_BE_EMPTY', function (done) {
            const body = {
                name: `Sublocation ${UID()}`,
                location_uid: '',
            };
            post('/api/v3/sublocations', body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"location_uid":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail create sublocation with fail with fake "location_uid"', function (done) {
            const fakeUid = UID();
            const body = { name: `Sublocation ${UID()}`, location_uid: fakeUid };
            post('/api/v3/sublocations', body)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Location with uid ${fakeUid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail create sublocation with fail with wrong elevation', function (done) {
            const body = {
                name: 'Sublocation Name',
                location_uid: location.uid,
                elevation: 12,
            };
            post('/api/v3/sublocations', body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"elevation":"NOT_MULTIPLE_TO 10"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should return list of sublocation', function (done) {
            get('/api/v3/sublocations')
                .expect(200)
                .then((response) => {
                    assert(response.body.sublocations.length === 9, `Wrong sublocations number`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should return specific sublocation', function (done) {
            get(`/api/v3/sublocations/${sublocation.uid}`)
                .expect(200)
                .then((response) => {
                    assert(response.body.sublocations.length === 1, `Wrong sublocation number`);
                    assert.deepStrictEqual(response.body.sublocations[0], sublocation, `Wrong sublocation object`);
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Update Sublocations', function () {
        it('should update sublocation with empty fields', function (done) {
            const body = {};
            put(`/api/v3/sublocations/${sublocation.uid}`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.name, sublocation.name, `Wrong sublocation "name" value`);
                    assert.strictEqual(response.body.id, sublocation.id, `Wrong sublocation "id" value`);
                    assert.strictEqual(
                        response.body.elevation,
                        sublocation.elevation,
                        `Wrong sublocation "elevation" value`
                    );
                    assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);
                    assert(
                        new Date(response.body.updated_at) > new Date(sublocation.updated_at),
                        `"updated_at" should be changed`
                    );
                    assert.strictEqual(
                        response.body.created_at,
                        sublocation.created_at,
                        `created_at should not be changed`
                    );
                    sublocation.name = body.name;
                    done();
                })
                .catch((err) => done(err));
        });

        it('should update sublocation name', function (done) {
            const body = {
                name: 'New Sublocation Name',
                elevation: 1000,
            };
            put(`/api/v3/sublocations/${sublocation.uid}`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.name, body.name, `Wrong sublocation "name" value`);
                    assert.strictEqual(response.body.id, sublocation.id, `Wrong sublocation "id" value`);
                    assert.strictEqual(response.body.elevation, body.elevation, `Wrong sublocation "elevation" value`);
                    assert(isISOTimeFormat(response.body.created_at), `Wrong sublocation "created_at" format`);
                    assert(isISOTimeFormat(response.body.updated_at), `Wrong sublocation "updated_at" format`);
                    assert(
                        new Date(response.body.updated_at) > new Date(sublocation.updated_at),
                        `"updated_at" should be changed`
                    );
                    assert.strictEqual(
                        response.body.created_at,
                        sublocation.created_at,
                        `created_at should not be changed`
                    );
                    sublocation.name = body.name;
                    done();
                })
                .catch((err) => done(err));
        });

        it('should return NotFound error on non existed uid', function (done) {
            const uid = UID();
            const body = {
                name: 'New Sublocation Name',
            };
            put(`/api/v3/sublocations/${uid}`, body)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Sublocation with uid ${uid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail update sublocation with empty "name"', function (done) {
            const body = {
                location_uid: location.uid,
                name: '',
            };
            put(`/api/v3/sublocations/${sublocation.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"name":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should fail update sublocation with fail with wrong elevation', function (done) {
            const body = {
                name: 'Sublocation Name',
                location_uid: location.uid,
                elevation: 12,
            };
            put(`/api/v3/sublocations/${sublocation.uid}`, body)
                .expect(422)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, '{"elevation":"NOT_MULTIPLE_TO 10"}', `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Delete Sublocations', function () {
        it('should return NotFound error on non existed uid', function (done) {
            const uid = UID();
            del(`/api/v3/sublocations/${uid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(message, `Sublocation with uid ${uid} not found`, `Wrong error message`);
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });

        it('should successfully delete sublocation', function (done) {
            del(`/api/v3/sublocations/${sublocation.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, sublocation.uid, `Wrong sublocation "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should not found removed sublocation', function (done) {
            get(`/api/v3/sublocations/${sublocation.uid}`)
                .expect(404)
                .then((response) => {
                    const { message, code } = response.body;
                    assert.deepStrictEqual(
                        message,
                        `Sublocation with uid ${sublocation.uid} not found`,
                        `Wrong error message`
                    );
                    assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Cascade Removal', function () {
        const node = {};
        const anchor = {};
        const nodeAssociation = {};
        const sublocationTwo = {};

        before(async function () {
            Object.assign(sublocationTwo, await createSublocation(`Sublocation ${UID()}`, location));
            Object.assign(node, await createNode('e4956ea5010b'));
            Object.assign(anchor, await createAnchor(`Anchor ${UID()}`, sublocationTwo));
            Object.assign(nodeAssociation, await createNodeAssociation(anchor, node));
            anchor.node_mac = node.mac;
        });

        after(async function () {
            await del(`/api/v3/nodes/${node.mac}`);
        });

        it('should return list of node associations', checkEntities('node_assignments', [nodeAssociation]));

        it('should return list of anchors', checkEntities('anchors', [anchor]));

        it('should remove sublocation', function (done) {
            del(`/api/v3/sublocations/${sublocationTwo.uid}`)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(response.body.uid, sublocationTwo.uid, `Wrong sublocation "uid" value`);
                    done();
                })
                .catch((err) => done(err));
        });

        it('should return empty list of node associations', checkEntities('node_assignments', []));

        it('should return empty list of anchors', checkEntities('anchors', []));
    });

    describe.skip('ZMQ Messaging for Sublocation CRUD', function () {
        const zmqObserver = new ZmqObserver();
        const sublocationZmq = {};

        before(function () {
            zmqObserver.init();
        });
        after(function () {
            zmqObserver.destroy();
        });

        it('should send zmq created sublocation message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.CREATED, object: ZMQ_OBJECTS.SUBLOCATION };
            const body = {
                name: 'Sublocation Zmq Name',
                location_uid: location.uid,
            };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.id, 'ZMQ message has no id');
                assert(zmqMsg.data.uid, 'ZMQ message has no uid');
                assert(zmqMsg.data.name === body.name, 'ZMQ message has wrong sublocation name');
                assert(zmqMsg.data.location_uid === body.location_uid, 'ZMQ message has wrong location_uid prop');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                assert(isISOTimeFormat(zmqMsg.data.updated_at), 'Wrong ZMQ message updated_at value');
                done();
            });
            subscription
                .then(() => post(`/api/v3/sublocations`, body))
                .then((response) => {
                    Object.assign(sublocationZmq, response.body);
                });
        });
        it('should send zmq updated anchor message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.UPDATED, object: ZMQ_OBJECTS.SUBLOCATION };
            const body = {
                name: 'New sublocation name',
            };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.id === sublocationZmq.id, 'ZMQ message has no id');
                assert(zmqMsg.data.uid === sublocationZmq.uid, 'ZMQ message has no uid');
                assert(zmqMsg.data.name === body.name, 'ZMQ message has wrong sublocation name');
                assert(zmqMsg.data.location_uid === sublocationZmq.location_uid, 'ZMQ message has wrong location_uid');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                assert(isISOTimeFormat(zmqMsg.data.updated_at), 'Wrong ZMQ message updated_at value');
                done();
            });
            subscription.then(() => put(`/api/v3/sublocations/${sublocationZmq.uid}`, body));
        });
        it('should send zmq deleted sublocation message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.DELETED, object: ZMQ_OBJECTS.SUBLOCATION };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert.deepStrictEqual(zmqMsg.data, { uid: sublocationZmq.uid }, 'ZMQ message data wrong');
                done();
            });
            subscription.then(() => del(`/api/v3/sublocations/${sublocationZmq.uid}`));
        });
    });
});
