import makeRequestHandler from '../../../utils/makeRequestHandler.mjs';
import { TrackableGet, TrackableDelete, TrackableSave, TrackableUpdate } from '../../../../usecases/index.mjs';

export default {
    /**
     * @swagger
     * /api/v3/trackables:
     *   get:
     *     description: Retrieve trackables list
     *     tags:
     *       - Trackables API
     *     responses:
     *       '200':
     *         description: trackables
     *         content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/Trackables'
     *                  example:
     *                      trackables:
     *                          -
     *                              uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *                              name: John Doe
     *                              created_at: 2021-04-21T10:00:00.000Z
     *                              updated_at: 2021-04-21T10:00:00.000Z
     *                              position_reporting:
     *                                  interval: 0.25
     *                                  auto: true
     *                              heartbeat: ENABLED
     *                              tag_mount_height: 1000
     *                              tag_mac: E4956EA501FC
     *                              position:
     *                                  timestamp: 2021-04-20T08:51:02.415Z
     *                                  x: 1000
     *                                  y: 2000
     *                                  z: 1000
     *                                  sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     *                          -
     *                              uid: eJEG2Q2kRnSHJaBw1FlA53
     *                              name: John Doe
     *                              created_at: 2021-04-21T10:00:00.000Z
     *                              updated_at: 2021-04-21T10:00:00.000Z
     *                              position_reporting:
     *                                  interval: 0.25
     *                                  auto: true
     *                              heartbeat: ENABLED
     *                              tag_mount_height: 1000
     *                              tag_mac: E4956EA501FC
     *                              position:
     *                                  timestamp: 2021-04-20T08:51:02.415Z
     *                                  x: 1000
     *                                  y: 2000
     *                                  z: 1000
     *                                  sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     */
    getAll: makeRequestHandler(
        TrackableGet,
        () => ({}),
        (result, res) => {
            res.json(result);
        }
    ),

    /**
     * @swagger
     * /api/v3/trackables/{uid}:
     *   get:
     *     description: Retrieve single trackable
     *     tags:
     *       - Trackables API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: eJEG2Q2kRnSHJaBdHxQ3Pw
     *          description: trackable unique identifier
     *     responses:
     *       '200':
     *          description: trackables
     *          content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/Trackables'
     *                  example:
     *                      trackables:
     *                          -
     *                              uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *                              name: John Doe
     *                              created_at: 2021-04-21T10:00:00.000Z
     *                              updated_at: 2021-04-21T10:00:00.000Z
     *                              position_reporting:
     *                                  interval: 0.25
     *                                  auto: true
     *                              heartbeat: ENABLED
     *                              tag_mount_height: 1000
     *                              tag_mac: E4956EA501FC
     *                              position:
     *                                  timestamp: 2021-04-20T08:51:02.415Z
     *                                  x: 1000
     *                                  y: 2000
     *                                  z: 1000
     *                                  sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     *       '404':
     *          description: A trackable with the specified UID was not found.
     *          content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/NotFoundError'
     *                  example:
     *                      message: Trackable with uid ... not found
     *                      code: NOT_FOUND_ERROR
     */
    get: makeRequestHandler(
        TrackableGet,
        (req) => {
            return {
                uid: req.params && req.params.uid,
            };
        },
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/trackables:
     *   post:
     *     description: Create trackable
     *     tags:
     *       - Trackables API
     *     requestBody:
     *      content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/TrackableCreateBody'
     *              example:
     *                  name: John Doe
     *                  position_reporting:
     *                      interval: 0.25
     *                  tag_mount_height: 1000
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Trackable'
     *              example:
     *                  uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *                  name: John Doe
     *                  created_at: 2021-04-21T10:00:00.000Z
     *                  updated_at: 2021-04-21T10:00:00.000Z
     *                  position_reporting:
     *                      interval: 0.25
     *                      auto: true
     *                  heartbeat: ENABLED
     *                  tag_mount_height: 1000
     *                  tag_mac: null
     *                  position: null
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  No name specify:
     *                      value:
     *                          message: '{"name":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty name:
     *                      value:
     *                          message: '{"name":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Tag_mount_height should be integer:
     *                      value:
     *                          message: '{"tag_mount_height":"NOT_INTEGER"}'
     *                          code: VALIDATION_ERROR
     *                  Wrong heartbeat value, allowed ['enabled', 'disabled', 'ENABLED', 'DISABLED']:
     *                      value:
     *                          message: '{"heartbeat":"NOT_ALLOWED_VALUE"}'
     *                          code: VALIDATION_ERROR
     *                  Empty heartbeat:
     *                      value:
     *                          message: '{"heartbeat":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  No position_reporting specify:
     *                      value:
     *                          message: '{"position_reporting":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but no position_reporting.interval:
     *                      value:
     *                          message: '{"position_reporting":{"interval":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but position_reporting.interval no multiply 0.25:
     *                      value:
     *                          message: '{"position_reporting":{"interval":"NOT_MULTIPLE_TO 0.25"}}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but position_reporting.interval too low (min value 0):
     *                      value:
     *                          message: '{"position_reporting":{"interval":"TOO_LOW"}}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but position_reporting.interval should be decimal:
     *                      value:
     *                          message: '{"position_reporting":{"interval":"NOT_DECIMAL"}}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but no position_reporting.auto:
     *                      value:
     *                          message: '{"position_reporting":{"auto":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.x:
     *                      value:
     *                          message: '{"position":{"x":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.x should be integer:
     *                      value:
     *                          message: '{"position":{"x":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.y:
     *                      value:
     *                          message: '{"position":{"y":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.y should be integer:
     *                      value:
     *                          message: '{"position":{"y":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.z:
     *                      value:
     *                          message: '{"position":{"z":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.z should be integer:
     *                      value:
     *                          message: '{"position":{"z":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.sublocation_uid:
     *                      value:
     *                          message: '{"position":{"sublocation_uid":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but empty position.sublocation_uid:
     *                      value:
     *                          message: '{"position":{"sublocation_uid":"CANNOT_BE_EMPTY"}}'
     *                          code: VALIDATION_ERROR
     *       '404':
     *         description: Sublocation for trackable position was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: 'Sublocation with uid ... not found'
     *                  code: NOT_FOUND_ERROR
     */
    post: makeRequestHandler(
        TrackableSave,
        (req) => {
            return {
                uid: req.params && req.params.uid,
                name: req.body.name,
                position_reporting: req.body.position_reporting,
                heartbeat: req.body.heartbeat,
                tag_mount_height: req.body.tag_mount_height,
                position: req.body.position,
            };
        },
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/trackables/{uid}:
     *   put:
     *     description: Update trackable by uid
     *     tags:
     *       - Trackables API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: eJEG2Q2kRnSHJaBdHxQ3Pw
     *          description: trackable unique identifier
     *     requestBody:
     *      content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/TrackableUpdateBody'
     *              example:
     *                  name: John Doe New
     *                  tag_mount_height: 2000
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Trackable'
     *              example:
     *                  uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *                  name: John Doe New
     *                  created_at: 2021-04-21T10:00:00.000Z
     *                  updated_at: 2021-04-21T10:00:00.000Z
     *                  position_reporting:
     *                      interval: 0.25
     *                      auto: true
     *                  heartbeat: ENABLED
     *                  tag_mount_height: 2000
     *                  tag_mac: null
     *                  position: null
     *       '404':
     *         description: Trackable or sublocation for trackable position with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              examples:
     *                  Non existed trackable:
     *                      value:
     *                          message: Trackable with uid ... not found
     *                          code: NOT_FOUND_ERROR
     *                  Non existed sublocation:
     *                      value:
     *                          message: 'Sublocation with uid ... not found'
     *                          code: NOT_FOUND_ERROR
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  No uid specify:
     *                      value:
     *                          message: '{"uid":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty name:
     *                      value:
     *                          message: '{"name":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Tag_mount_height should be integer:
     *                      value:
     *                          message: '{"tag_mount_height":"NOT_INTEGER"}'
     *                          code: VALIDATION_ERROR
     *                  Wrong heartbeat value, allowed ['enabled', 'disabled', 'ENABLED', 'DISABLED']:
     *                      value:
     *                          message: '{"heartbeat":"NOT_ALLOWED_VALUE"}'
     *                          code: VALIDATION_ERROR
     *                  Empty heartbeat:
     *                      value:
     *                          message: '{"heartbeat":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but no position_reporting.interval:
     *                      value:
     *                          message: '{"position_reporting":{"interval":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but position_reporting.interval no multiply 0.25:
     *                      value:
     *                          message: '{"position_reporting":{"interval":"NOT_MULTIPLE_TO 0.25"}}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but position_reporting.interval too low (min value 0):
     *                      value:
     *                          message: '{"position_reporting":{"interval":"TOO_LOW"}}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but position_reporting.interval should be decimal:
     *                      value:
     *                          message: '{"position_reporting":{"interval":"NOT_DECIMAL"}}'
     *                          code: VALIDATION_ERROR
     *                  Position_reporting specify but no position_reporting.auto:
     *                      value:
     *                          message: '{"position_reporting":{"auto":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.x:
     *                      value:
     *                          message: '{"position.x":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.x should be integer:
     *                      value:
     *                          message: '{"position":{"x":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.y:
     *                      value:
     *                          message: '{"position":{"y":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.y should be integer:
     *                      value:
     *                          message: '{"position":{"y":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.z:
     *                      value:
     *                          message: '{"position":{"z":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.z should be integer:
     *                      value:
     *                          message: '{"position":{"z":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.sublocation_uid:
     *                      value:
     *                          message: '{"position":{"sublocation_uid":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but empty position.sublocation_uid:
     *                      value:
     *                          message: '{"position":{"sublocation_uid":"CANNOT_BE_EMPTY"}}'
     *                          code: VALIDATION_ERROR
     */
    put: makeRequestHandler(
        TrackableUpdate,
        (req) => {
            return {
                uid: req.params && req.params.uid,
                name: req.body.name,
                position_reporting: req.body.position_reporting,
                heartbeat: req.body.heartbeat,
                tag_mount_height: req.body.tag_mount_height,
                position: req.body.position,
            };
        },
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/trackables/{uid}:
     *   delete:
     *     description: Remove trackable by uid
     *     tags:
     *       - Trackables API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: eJEG2Q2kRnSHJaBdHxQ3Pw
     *          description: trackable unique identifier
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  type: object
     *                  properties:
     *                      uid:
     *                          type: string
     *              example:
     *                  uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *       '404':
     *         description: Trackable with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Trackable with uid ... not found
     *                  code: NOT_FOUND_ERROR
     */
    del: makeRequestHandler(
        TrackableDelete,
        (req) => ({
            uid: req.params && req.params.uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
};
