import ModelBase from '../ModelBase.mjs';
import { NotFoundError, ValidationError } from '../errors/index.mjs';
import { UID } from '../../utils/index.mjs';
import { formatResponse, formatResponseList } from './formatters.mjs';

/**
 * @swagger
 * components:
 *  schemas:
 *      LsbScheduleCreateBody:
 *          type: object
 *          properties:
 *              location_uid:
 *                  type: string
 *                  required: true
 *                  description: Unique identifier for the location
 *              name:
 *                  type: string
 *                  required: true
 *                  description: LSB schedule name
 *              content:
 *                  type: array
 *                  required: true
 *                  $ref: '#/components/schemas/LsbScheduleContentItem'
 *                  description: List of LSB schedule items
 *      LsbScheduleUpdateBody:
 *          type: object
 *          properties:
 *              name:
 *                  type: string
 *                  required: true
 *                  description: LSB schedule name
 *              content:
 *                  type: array
 *                  required: true
 *                  $ref: '#/components/schemas/LsbScheduleContentItem'
 *                  description: List of LSB schedule items
 *      LsbScheduleContentItem:
 *          type: object
 *          properties:
 *              source:
 *                  type: string
 *              destination:
 *                  type: string
 *              slot:
 *                  type: number
 *              section:
 *                  type: number
 *              dimension:
 *                  type: string
 *              boundary:
 *                  type: bool
 *              margin:
 *                  type: number
 *      LsbSchedule:
 *          type: object
 *          properties:
 *              uid:
 *                  type: string
 *                  description: Schedule unique identifier
 *              location_uid:
 *                  type: string
 *                  description: Unique identifier for the location
 *              created_at:
 *                  type: string (ISO 8601)
 *                  description: The timestamp when the schedule was created
 *              updated_at:
 *                  type: string (ISO 8601)
 *                  description: The timestamp when the schedule was updated
 *              name:
 *                  type: string
 *                  description: Schedule name
 *              content:
 *                  type: array
 *                  $ref: '#/components/schemas/LsbScheduleContentItem'
 *                  description: List of LSB schedule items
 *      LsbSchedules:
 *          type: object
 *          properties:
 *              lsb_schedules:
 *                  type: array
 *                  items:
 *                      $ref: '#/components/schemas/LsbSchedule'
 */
export class NetworkLsbSchedule extends ModelBase {
    /**
     * @property {String|null}
     */
    static object = 'lsb_schedule';

    /**
     * @method
     * @return {Promise<Object>}
     */
    async save(params) {
        const now = new Date().toISOString();
        const location = await this.repository.location.findByUid(params.location_uid);

        if (!location) {
            throw new NotFoundError(`Location with uid ${params.location_uid} not found`);
        }

        const lsbSchedule = {
            uid: UID(),
            created_at: now,
            updated_at: now,
            ...params,
            content: JSON.stringify(params.content),
        };

        await this.repository.networkLsbSchedule.save(lsbSchedule);

        params.uid = lsbSchedule.uid;
        params.created_at = lsbSchedule.created_at;
        params.updated_at = lsbSchedule.updated_at;
        params.content = params.content.map((contentItem) => ({
            ...contentItem,
            boundary: !!contentItem.boundary,
        }));

        return params;
    }

    /**
     * @method
     * @param {object} params - method params
     * @property {string} params.uid - Lsb schedule unique id
     * @property {string} params.location_uid - Location unique id
     * @return {Promise<Object>}
     */
    async load(params) {
        const { uid, location_uid } = params;
        if (location_uid) {
            const networkLsbSchedule = (await this.repository.networkLsbSchedule.findByLocationUid(location_uid)) || [];
            return { lsb_schedules: formatResponseList(networkLsbSchedule) };
        } else if (uid) {
            const networkLsbSchedule = await this.repository.networkLsbSchedule.findByUid(uid);
            if (!networkLsbSchedule) {
                throw new NotFoundError(`Network schedule with uid ${uid} not found`);
            }
            return { lsb_schedules: [formatResponse(networkLsbSchedule)] };
        } else {
            const networkLsbSchedules = (await this.repository.networkLsbSchedule.findAll()) || [];
            return { lsb_schedules: formatResponseList(networkLsbSchedules) };
        }
    }

    /**
     * @method
     * @param {object} params - method params
     * @property {string} params.uid - Lsb schedule unique id
     * @return {Promise<Object>}
     */
    async remove(params) {
        const { uid } = params;
        const networkLsbSchedule = await this.repository.networkLsbSchedule.findByUid(uid);
        if (!networkLsbSchedule) {
            throw new NotFoundError(`Network schedule with uid ${uid} not found`);
        }
        const networkSettings = await this.repository.networkSetting.findByUid(networkLsbSchedule.location_uid);
        const locationUid = networkLsbSchedule.location_uid;
        if (networkSettings && networkSettings.value.active_schedule_uid === uid) {
            throw new ValidationError(
                `This schedule is set as active for location ${locationUid}. Please update location settings first.`
            );
        }
        await this.repository.networkLsbSchedule.del(networkLsbSchedule);

        return formatResponse(networkLsbSchedule);
    }

    /**
     * @method
     * @param {object} params - method params
     * @property {string} params.uid - LSB schedule unique id
     * @property {object} params.name - LSB schedule name
     * @property {object} params.content - List of LSB schedule items
     * @property {string} params.content.source - Source anchor UID
     * @property {string} params.content.destination - Destination anchor UID
     * @property {number} params.content.slot - Slot number: 0..254
     * @property {number} params.content.section - Section number: 0 (“A”), 1 (”B”), for 2D dimension only 0
     * @property {string} params.content.dimension - Dimension type: 0D, 1D, 2D
     * @property {boolean} params.content.boundary - Boundary: yes/no
     * @property {number} params.content.margin - Margin (mm): 0 .. 50 000, multiple 250
     * @return {Promise<Object>}
     */
    async update(params) {
        const { uid } = params;

        const networkLsbSchedule = await this.repository.networkLsbSchedule.findByUid(uid);
        if (!networkLsbSchedule) {
            throw new NotFoundError(`Network schedule with uid ${uid} not found`);
        }

        const lsbSchedule = {
            updated_at: new Date().toISOString(),
            ...params,
        };
        if (lsbSchedule.content) {
            lsbSchedule.content = JSON.stringify(params.content);
        }

        await this.repository.networkLsbSchedule.save(lsbSchedule);

        return formatResponse(await this.repository.networkLsbSchedule.findByUid(uid));
    }
}
