import ModelBase from '../ModelBase.mjs';
import { NotFoundError } from '../errors/index.mjs';
import { prepareSettingToSave, formatSettingResponse } from './formatters.mjs';

/**
 * @swagger
 * components:
 *  schemas:
 *      NetworkSettingCreateBody:
 *          type: object
 *          properties:
 *              location_uid:
 *                  type: string
 *                  required: true
 *              value:
 *                  type: object
 *                  required: true
 *                  properties:
 *                      active_schedule_uid:
 *                          type: string
 *                          required: true
 *                      masters:
 *                          type: array
 *                          required: true
 *                          items:
 *                              type: object
 *                              properties:
 *                                  anchor_uid:
 *                                      type: string
 *                                  sfd:
 *                                      type: number
 *                                  auto_sync:
 *                                      type: boolean
 *                      distributors:
 *                          type: array
 *                          required: true
 *                          items:
 *                              type: object
 *                              properties:
 *                                  anchor_uid:
 *                                      type: string
 *                                  master_uid:
 *                                      type: string
 *      NetworkSettingUpdateBody:
 *          type: object
 *          properties:
 *              value:
 *                  type: object
 *                  required: true
 *                  properties:
 *                      active_schedule_uid:
 *                          type: string
 *                          required: true
 *                      masters:
 *                          type: array
 *                          required: true
 *                          items:
 *                              type: object
 *                              properties:
 *                                  anchor_uid:
 *                                      type: string
 *                                  sfd:
 *                                      type: number
 *                                  auto_sync:
 *                                      type: boolean
 *                      distributors:
 *                          type: array
 *                          required: true
 *                          items:
 *                              type: object
 *                              properties:
 *                                  anchor_uid:
 *                                      type: string
 *                                  master_uid:
 *                                      type: string
 *      NetworkSetting:
 *          type: object
 *          properties:
 *              location_uid:
 *                  type: string
 *                  description: Unique id of location.
 *              created_at:
 *                  type: string
 *                  description: Timestamp in ISO 8601 format of when network setting was created.
 *              updated_at:
 *                  type: string
 *                  description: Timestamp in ISO 8601 format of when network setting was lst updated.
 *              value:
 *                  type: object
 *                  description: Network settings value.
 *                  required: true
 *                  properties:
 *                      active_schedule_uid:
 *                          type: string
 *                          description: Active schedule unique identifier.
 *                          required: true
 *                      masters:
 *                          type: array
 *                          description: List of masters within the the location.
 *                          required: true
 *                          items:
 *                              type: object
 *                              properties:
 *                                  anchor_uid:
 *                                      type: string
 *                                      description: Anchor unique identifier.
 *                                  sfd:
 *                                      type: number
 *                                      description: Super frame duration 1000 (1 Hz), 500 (2 Hz), 250 (4 Hz).
 *                                  auto_sync:
 *                                      type: boolean
 *                                      description: The flag that indicates synchronization regime.
 *                      distributors:
 *                          type: array
 *                          description: List of distributors within the the location.
 *                          required: true
 *                          items:
 *                              type: object
 *                              properties:
 *                                  anchor_uid:
 *                                      type: string
 *                                      description: Anchor unique identifier.
 *                                  master_uid:
 *                                      type: string
 *                                      description: Anchor unique identifier.
 *
 *      NetworkSettings:
 *          type: object
 *          properties:
 *              settings:
 *                  type: array
 *                  items:
 *                      $ref: '#/components/schemas/NetworkSetting'
 */

export class NetworkSetting extends ModelBase {
    /**
     * @typedef {Class} NetworkSetting
     */

    /**
     * @typedef {Object} NetworkSettingCreateBody
     * @property {String} location_uid
     * @property {Object} value
     * @property {String} value.active_schedule_uid
     * @property {Array<{anchor_uid: String, sfd: Number, auto_sync: Boolean}>} value.masters
     * @property {Array<{anchor_uid: String, master_uid: String}>} value.distributors
     */

    /**
     * @property {String|null}
     */
    static object = 'network_setting';

    /**
     * @param {NetworkSettingCreateBody} params  - data to save
     * @returns {Promise<Object>}
     */
    async save(params) {
        const location = await this.repository.location.findByUid(params.location_uid);
        const lsbSchedule = await this.repository.networkLsbSchedule.findByUid(params.value.active_schedule_uid);
        if (!lsbSchedule) {
            throw new NotFoundError(`LSB schedule with uid ${params.value.active_schedule_uid} not found`);
        }
        if (!location) {
            throw new NotFoundError(`Location with uid ${params.location_uid} not found`);
        }
        const networkSettings = await this.repository.networkSetting.findByUid(params.location_uid);
        if (networkSettings) {
            await this.repository.networkSetting.deleteByUid(params.location_uid);
        }

        const now = new Date().toISOString();
        const saveData = { ...params, created_at: networkSettings ? networkSettings.created_at : now, updated_at: now };

        await this.repository.networkSetting.save(prepareSettingToSave(saveData));

        return formatSettingResponse(saveData);
    }

    /**
     * @method
     * @param {Object} params - request params
     * @param {String} params.location_uid - location uid
     * @return {Promise<Object>}
     */
    async load(params) {
        const { location_uid } = params;
        if (location_uid) {
            const networkSetting = await this.repository.networkSetting.findByUid(location_uid);
            if (!networkSetting) {
                throw new NotFoundError(`Network settings for location with uid ${location_uid} not found`);
            }
            return {
                settings: [formatSettingResponse(networkSetting)],
            };
        }
        const networkSettings = (await this.repository.networkSetting.findAll()) || [];

        return {
            settings: networkSettings.map((setting) => formatSettingResponse(setting)),
        };
    }

    /**
     * @method
     * @param {Object} params - request parameters
     * @param {String} params.location_uid - location uid
     * @return {Promise<Object>}
     */
    async remove(params) {
        const { location_uid } = params;
        const location = await this.repository.location.findByUid(location_uid);
        if (!location) {
            throw new NotFoundError(`Location with uid ${location_uid} not found`);
        }
        const networkSettings = await this.repository.networkSetting.findByUid(location_uid);
        if (!networkSettings) {
            throw new NotFoundError(`Network settings for location with uid ${location_uid} not found`);
        }
        await this.repository.networkSetting.deleteByUid({ location_uid });

        return formatSettingResponse(networkSettings);
    }
}
