import ModelBase from '../ModelBase.mjs';
import { UID } from '../../utils/index.mjs';
import { NodeId } from '../id/index.mjs';
import { formatNode } from './formatters.mjs';
import { NotFoundError, ValidationError } from '../errors/index.mjs';
import { NODE_TYPE } from '../../system/constants.mjs';

export class NodeBase extends ModelBase {
    constructor(type) {
        super();
        this.type = type;
        this.entityName = type === NODE_TYPE.TO ? 'Tag' : 'Node';
    }

    /**
     *
     * @param {Object} data  - data to save
     * @param {String} data.mac  - node MAC address
     * @returns {Promise<Object>}
     */
    async save(data) {
        const { mac } = data;
        const nodeByMac = await this.repository.node.findByMac(mac);
        if (nodeByMac && nodeByMac.type) {
            throw new ValidationError(`${this.entityName} with MAC ${mac} already exist`);
        }
        const now = new Date().toISOString();
        const uid = nodeByMac ? nodeByMac.uid : UID();
        const nodeId = nodeByMac ? nodeByMac.node_id : await new NodeId().nextValue();

        const node = {
            uid,
            mac,
            node_id: nodeId,
            last_heard: null,
            created_at: now,
            updated_at: now,
            cbid: null,
            type: this.type,
        };

        await this.repository.node.save(node);

        return formatNode(node);
    }

    /**
     *
     * @param {Object} options
     * @param {String} options.mac
     * @returns {Promise<Array<Object>>}
     */
    async load(options = {}) {
        const { mac } = options;
        const filterByType = (node) => node.type === this.type;
        if (!mac) {
            return (await this.repository.node.findAll()).filter(filterByType).map(formatNode);
        }
        const node = await this.repository.node.findByMac(mac);
        if (node && node.type === this.type) {
            return [formatNode(node)];
        }
        throw new NotFoundError(`${this.entityName} with MAC ${mac} not found`);
    }

    /**
     @typedef WithChildren
     @type {Object}
     @property {Object} node - Node/Tag.
     @property {Object} association - Node/Tag association.
     */
    /**
     * @method
     * @param {Object} params - options for read node/tag in repository
     * @param {String} params.mac - node/tag MAC address
     * @return {Promise<WithChildren>}
     */
    async loadWithAssociation(params) {
        const { mac } = params;
        const node = await this.repository.node.findByMac(mac);
        if (!node || node.type !== this.type) {
            throw new NotFoundError(`${this.entityName} with MAC ${mac} not found`);
        }
        const association = await this.repository.association.findByMac(mac);
        return { node, association };
    }

    /**
     *
     * @param {Object} params
     * @property {String} params.mac
     * @returns {Promise<Object>}
     */
    async remove(params) {
        const { mac } = params;
        const node = await this.repository.node.findByMac(mac);
        if (!node || node.type !== this.type) {
            throw new NotFoundError(`${this.entityName} with MAC ${mac} not found`);
        }

        // todo: add transaction
        await this.repository.node.del(node);
        await new NodeId().remove(node.node_id);

        return formatNode(node);
    }
}
