import ModelBase from '../ModelBase.mjs';
import { NotFoundError } from '../errors/index.mjs';
import { formatTrackableResponse, formatTrackableResponseList } from './formatters.mjs';
import { UID } from '../../utils/index.mjs';

/**
 * @swagger
 * components:
 *  schemas:
 *      TrackableCreateBody:
 *          type: object
 *          properties:
 *              name:
 *                  type: string
 *                  required: true
 *              position_reporting:
 *                  type: object
 *                  required: true
 *                  properties:
 *                      interval:
 *                          type: number
 *                          required: true
 *                      auto:
 *                          type: boolean
 *                          required: true
 *              heartbeat:
 *                  type: string
 *                  enum: [enabled, disabled]
 *                  default: enabled
 *                  required: false
 *              tag_mount_height:
 *                  type: number
 *                  required: false
 *                  default: 0
 *              position:
 *                  type: object
 *                  required: false
 *                  properties:
 *                      x:
 *                          type: number
 *                          required: true
 *                      y:
 *                          type: number
 *                          required: true
 *                      z:
 *                          type: number
 *                          required: true
 *                      sublocation_uid:
 *                          type: string
 *                          required: true
 *      TrackableUpdateBody:
 *          type: object
 *          properties:
 *              name:
 *                  type: string
 *                  required: false
 *              position_reporting:
 *                  type: object
 *                  required: false
 *                  properties:
 *                      interval:
 *                          type: number
 *                          required: true
 *                      auto:
 *                          type: boolean
 *                          required: true
 *              heartbeat:
 *                  type: string
 *                  enum: [enabled, disabled]
 *                  required: false
 *              tag_mount_height:
 *                  type: number
 *                  required: false
 *              position:
 *                  type: object
 *                  required: false
 *                  properties:
 *                      x:
 *                          type: number
 *                          required: true
 *                      y:
 *                          type: number
 *                          required: true
 *                      z:
 *                          type: number
 *                          required: true
 *                      sublocation_uid:
 *                          type: string
 *                          required: true
 *      Trackable:
 *          type: object
 *          properties:
 *              uid:
 *                  type: string
 *                  description: Unique id of trackable.
 *              name:
 *                  type: string
 *                  description: Trackable name.
 *              created_at:
 *                  type: string
 *                  description: Timestamp of trackable create.
 *              updated_at:
 *                  type: string
 *                  description: Timestamp of trackable update.
 *              position_reporting:
 *                  type: object
 *                  description: JSON object determined positioning configuration.
 *                  properties:
 *                      interval:
 *                          type: number
 *                          description: The interval, in seconds, between each position reporting (minimal value equal to 0, step equal to 0.25).
 *                      auto:
 *                          type: boolean
 *                          description: The flag that indicates positioning regime.
 *              tag_mount_height:
 *                  type: number
 *                  description: Height of the tag mounting on the trackable, in millimeters.
 *              heartbeat:
 *                  type: string
 *                  description: The flag that indicates do the trackable send a heartbeats(enabled, disabled).
 *              tag_mac:
 *                  type: string
 *                  description: The MAC address of the tag assigned to the trackable.
 *              position:
 *                  type: object
 *                  description: JSON object indicates the last known trackable position.
 *                  properties:
 *                      timestamp:
 *                          type: string
 *                          description: Timestamp of the position in ISO 8601 format.
 *                      x:
 *                          type: number
 *                          description: The X coordinate of the trackable position.
 *                      y:
 *                          type: number
 *                          description: The Y coordinate of the trackable position.
 *                      z:
 *                          type: number
 *                          description: The Z coordinate of the trackable position.
 *                      sublocation_uid:
 *                          type: string
 *                          description: Unique identifier of the position sublocation.
 *
 *      Trackables:
 *          type: object
 *          properties:
 *              trackables:
 *                  type: array
 *                  items:
 *                      $ref: '#/components/schemas/Trackable'
 */

export class Trackable extends ModelBase {
    /**
     * @typedef {Class} Trackable
     */

    /**
     * @property {String|null}
     */
    static object = 'trackable';

    /**
     * @param {Object} params  - data to save
     * @returns {Promise<Object>}
     */
    async save(params) {
        if (params.position) {
            const sublocation = await this.repository.sublocation.findByUid(params.position.sublocation_uid);

            if (!sublocation) {
                throw new NotFoundError(`Sublocation with uid ${params.position.sublocation_uid} not found`);
            }
        }
    
        if (params.heartbeat) {
            params.heartbeat = params.heartbeat.toUpperCase();
        }
        
        const now = new Date().toISOString();
        const trackable = {
            uid: UID(),
            created_at: now,
            updated_at: now,
            ...params,
        };
        await this.repository.trackable.save(trackable);
        return formatTrackableResponse(trackable);
    }

    /**
     * @method
     * @param {Object} params - save data
     * @return {Promise<Object>}
     */
    async load(params) {
        const { uid } = params;
        if (!uid) {
            const dbTrackables = (await this.repository.trackable.findAll()) || [];

            const tagAssociations = (await this.repository.association.findAll()) || [];
            const trackablesWithTagMac = dbTrackables.reduce((memo, trackable) => {
                const tagAssociation = tagAssociations.find((assoc) => assoc.trackable_uid === trackable.uid);
                if (tagAssociation) {
                    memo.push({ ...trackable, tag_mac: tagAssociation.mac });
                } else {
                    memo.push(trackable);
                }
                return memo;
            }, []);

            return formatTrackableResponseList(trackablesWithTagMac);
        }
        const trackable = await this.repository.trackable.findByUid(uid);
        if (!trackable) {
            throw new NotFoundError(`Trackable with uid ${uid} not found`);
        }
        const tagAssociation = await this.repository.association.findByObjUid(trackable.uid);
        if (tagAssociation) {
            trackable.tag_mac = tagAssociation.mac;
        }
        return [formatTrackableResponse(trackable)];
    }

    /**
     @typedef TrackableWithChildren
     @type {Object}
     @property {Object} trackable - Trackable.
     @property {Object} tagAssociation - Tag association.
     */
    /**
     * @method
     * @param {Object} params - options for read trackable in repository
     * @param {String} params.uid - trackable unique identifier
     * @return {Promise<TrackableWithChildren>}
     */
    async loadWithAssociation(params) {
        const { uid } = params;
        const trackable = await this.repository.trackable.findByUid(uid);
        if (!trackable) {
            throw new NotFoundError(`Trackable with uid ${uid} not found`);
        }
        const tagAssociation = await this.repository.association.findByObjUid(uid);
        return { trackable, tagAssociation };
    }

    /**
     * @method
     * @param {Object} params - save data
     * @return {Promise<Object>}
     */
    async remove(params) {
        const { uid } = params;
        const trackable = await this.repository.trackable.findByUid(uid);
        if (!trackable) {
            throw new NotFoundError(`Trackable with uid ${uid} not found`);
        }
        return await this.repository.trackable.deleteByUid(uid);
    }

    /**
     * @method
     * @param {Object} params - save data
     * @return {Promise<Object>}
     */
    async update(params) {
        const { uid } = params;
        const trackable = await this.repository.trackable.findByUid(uid);
        if (!trackable) {
            throw new NotFoundError(`Trackable with uid ${uid} not found`);
        }

        if (params.position) {
            const sublocation = await this.repository.sublocation.findByUid(params.position.sublocation_uid);

            if (!sublocation) {
                throw new NotFoundError(`Sublocation with uid ${params.position.sublocation_uid} not found`);
            }
        }
        
        if (params.heartbeat) {
            params.heartbeat = params.heartbeat.toUpperCase();
        }

        await this.repository.trackable.save({ ...params, updated_at: new Date().toISOString() });

        const updatedTrackable = await this.repository.trackable.findByUid(uid);

        const tagAssociation = await this.repository.association.findByObjUid(trackable.uid);
        if (tagAssociation) {
            updatedTrackable.tag_mac = tagAssociation.mac;
        }

        return formatTrackableResponse(updatedTrackable);
    }
}
