import flat from 'flat';
import { RepoError } from '../models/index.mjs';
import RepoRedisBase from '../RepoRedisBase.mjs';

const unflatten = flat.unflatten;


/**
 * @typedef PlatformAnchor {Object}
 * @property name {String}
 * @property created_at {String}
 * @property updated_at {String}
 * @property uid {String}
 * @property bridge {Boolean}
 * @property position {Object}
 * @property position.x {Number}
 * @property position.y {Number}
 * @property position.z {Number}
 * @property position.sublocation_uid {String}
 */

export class AnchorRepository extends RepoRedisBase {
    /**
     * @typedef {Class} AnchorRepository
     */

    /**
     * @param {Object} options - repository options
     * @param {Object} options.repositoryInstance - database connection
     * @param {String} options.prefix - database prefix
     */
    constructor(options) {
        const { prefix, repositoryInstance } = options;

        super(repositoryInstance, `${prefix}:anchor`);
    }
    
    /**
     *
     * @param obj {PlatformAnchor}
     * @returns {Promise<{position}|Object|Object>}
     */
    async save(obj) {
        if (!obj.position) {
            return super.save(obj);
        } else if (typeof obj.position === 'object') {
            const tko = {
                ...obj,
                position: JSON.stringify(obj.position)
            }
            await super.save(tko);
            return obj;

        } else if (typeof obj.position === 'string') {
            return super.save(obj);
        }
    }
    /**
     * @method
     * @param {String} uid - anchor uid
     * @return {Promise<Object>} returns anchor
     */
    async   findByUid(uid) {
        if (!uid) {
            throw new RepoError('specify uid in findByUid method');
        }
        const flatObject = await this.repoInstance.hgetall(`${this.document}:${uid}`);
        if (!flatObject) {
            return null;
        }
        return this.deflate(flatObject);
    }
    deflate(flatObject) {
        if (flatObject.position) {
            const position = JSON.parse(flatObject.position);
            delete flatObject.position;
            return {
                position,
                ...unflatten(flatObject),
            };
        } else {
            return unflatten(flatObject);
        }
    }
    /**
     * @method
     * @return {Promise<Array<Object>>} returns entities list
     */
    async findAll() {
        return (await super.findAll() || []).map(flatObject => this.deflate(flatObject));
    }
}
