import flat from 'flat';
import RepoRedisBase from '../RepoRedisBase.mjs';
import { RepoError } from '../models/index.mjs';

const unflatten = flat.unflatten;
const flatten = flat.flatten;

export class NetworkSettingRepository extends RepoRedisBase {
    assocResource;
    /**
     * @typedef {Class} NetworkSettingRepository
     */

    /**
     * @param {Object} options - repository options
     * @param {Object} options.repositoryInstance - database connection
     * @param {String} options.prefix - database prefix
     */
    constructor(options) {
        const { prefix, repositoryInstance } = options;
        super(repositoryInstance, `${prefix}:network_setting`);
    }

    /**
     * @method
     * @param {String} uid - location uid
     * @return {Promise<Object>} returns network setting
     */
    async findByUid(uid) {
        if (!uid) {
            throw new RepoError('specify uid in findByUid method');
        }
        const flatObject = await this.repoInstance.hgetall(`${this.document}:${uid}`);
        if (!flatObject) {
            return null;
        }
        return unflatten(flatObject);
    }

    /**
     * @method
     * @param {Object} obj - data to save in DB
     * @return {Promise<Object>} returns data saved in DB
     */
    async save(obj) {
        if (!this.document) {
            throw new RepoError(`save: no document`);
        }
        if (!obj || typeof obj !== 'object') {
            throw new RepoError(`save: object saving is aborted due to object format: ${JSON.stringify(obj)}`);
        }
        const fields = [];
        for (const [key, value] of Object.entries(flatten(obj))) {
            if (value || typeof value === 'number') {
                // do not save empty values
                fields.push(key, value);
            }
        }

        await this.repoInstance.hmset(`${this.document}:${obj.location_uid}`, ...fields);
        return obj;
    }

    /**
     * @method
     * @param {Object} params - parameters
     * @param {String} params.location_uid - location unique identifier
     * @return {Promise<Object>} returns result of key removing
     */
    async deleteByUid(params) {
        return this.repoInstance.del(`${this.document}:${params.location_uid}`);
    }
}
